package provide DS9 1.0

proc Bin {value} {
    global bin
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) bin factor $value
	set bin(factor) [$current(frame) get bin factor]

	UpdateBinDialog
	UpdateContourMenu
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc BinAbout {x y} {
    global bin
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) bin about $x $y physical

	UpdateBinDialog
	UpdateContourMenu
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc BinCols {x y} {
    global bin
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) bin cols "$x" "$y"

	UpdateBinDialog
	UpdateContourMenu
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc BinFilter {str} {
    global bin
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) bin filter "\{$str\}"

	UpdateBinDialog
	UpdateContourMenu
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc BinToFit {} {
    global current
    global bin

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) bin to fit
	set bin(factor) [$current(frame) get bin factor]

	UpdateBinDialog
	UpdateContourMenu
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc ChangeBinFactor {} {
    global bin
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) bin factor to $bin(factor)

	UpdateBinDialog
	UpdateContourMenu
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc ChangeBinFunction {} {
    global bin
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) bin function $bin(function)

	UpdateBinDialog
	UpdateContourMenu
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc ChangeBinBufferSize {} {
    global bin
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) bin buffer size $bin(buffersize)

	UpdateBinDialog
	UpdateContourMenu
	UpdateGraphXAxis
	UnsetWatchCursor
    }
}

proc BinDialog {} {
    global ds9
    global bindialog
    global menu

    # see if we already have a window visible

    if [winfo exist $bindialog(top)] {
	return
    }

    set w $bindialog(top)
    set title "Binning Parameters"

    # create the window

    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW PanZoomDestroyDialog

    $w configure -menu $bindialog(mb)

    menu $bindialog(mb) -tearoff 0
    $bindialog(mb) add cascade -label File -menu $bindialog(mb).file
    $bindialog(mb) add cascade -label Edit -menu $bindialog(mb).edit

    menu $bindialog(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $bindialog(mb).file add command -label "Apply" -command BinApplyDialog
    $bindialog(mb).file add separator
    $bindialog(mb).file add command -label "Close" -command BinDestroyDialog

    menu $bindialog(mb).edit -tearoff 0 -selectcolor $menu(selectcolor)
    $bindialog(mb).edit add command -label "Cut" -command BinCutDialog
    $bindialog(mb).edit add command -label "Copy" -command BinCopyDialog
    $bindialog(mb).edit add command -label "Paste" -command BinPasteDialog
    $bindialog(mb).edit add command -label "Clear" -command BinClearDialog

    frame $w.center -relief groove -borderwidth 2
    frame $w.param -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.center $w.param $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.center.title1 -text "Bin Factor"
    entry $w.center.factor -textvariable bindialog(factor) \
	-width 10 -relief sunken -bd 2
    label $w.center.title2 -text "Bin Center"
    entry $w.center.x -textvariable bindialog(x) -width 10 \
	-relief sunken -bd 2
    entry $w.center.y -textvariable bindialog(y) -width 10 \
	-relief sunken -bd 2
    label $w.center.title3 -text "physical"
    label $w.center.dummy -width 7

    checkbutton $w.center.auto -text "or about center" -variable bindialog(auto)

    grid rowconfigure $w.center 0 -pad 4
    grid rowconfigure $w.center 1 -pad 4
    grid rowconfigure $w.center 2 -pad 4

    grid $w.center.title1 -row 0 -column 0 -padx 4 -sticky w
    grid $w.center.factor -row 0 -column 1 -padx 4 -sticky w
    grid $w.center.title2 -row 1 -column 0 -padx 4 -sticky w
    grid $w.center.x -row 1 -column 1 -padx 4 -sticky w
    grid $w.center.y -row 1 -column 2 -padx 4 -sticky w
    grid $w.center.title3 -row 1 -column 3 -padx 4 -sticky w
    grid $w.center.dummy -row 1 -column 4
    grid $w.center.auto -row 2 -column 1 -padx 4 -sticky w -columnspan 3

    label $w.param.coltitle -text "Bin Columns"
    menubutton $w.param.xcol -textvariable bindialog(xcol) \
	-menu $w.param.xcol.m -relief raised -bd 2 -width 10
    menubutton $w.param.ycol -textvariable bindialog(ycol) \
	-menu $w.param.ycol.m -relief raised -bd 2 -width 10

    label $w.param.filtertitle -text "Bin Filter"
    entry $w.param.filter -textvariable bindialog(filter) \
	-width 40 -relief sunken -bd 2

    grid rowconfigure $w.param 0 -pad 4
    grid rowconfigure $w.param 1 -pad 4

    grid $w.param.coltitle -row 0 -column 0 -padx 4 -sticky w
    grid $w.param.xcol -row 0 -column 1 -padx 4 -sticky w
    grid $w.param.ycol -row 0 -column 2 -padx 4 -sticky w

    grid $w.param.filtertitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.param.filter -row 1 -column 1 -columnspan 2 -padx 4 -sticky w

    button $w.buttons.apply -text "Apply" -command BinApplyDialog
    button $w.buttons.close -text "Close" -command BinDestroyDialog
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    $w.center.factor select range 0 end

    set bindialog(auto) 0
    UpdateBinDialog
}

proc PopUp {b m l} {
    destroy $m

    menu $m -tearoff 0
    for {set i 0} {$i<[llength $l]} {incr i} {
	$m add command -label [lindex $l $i] \
	    -command "global bindialog;set $b [lindex $l $i]"
    }
}

proc UpdateBinDialog {} {
    global bindialog
    global current

    if {[winfo exist $bindialog(top)]} {
	if {$current(frame) != ""} {
	    set w $bindialog(top)
	    set c [$current(frame) get bin cols]
	    set bindialog(xcol) [lindex $c 0]
	    set bindialog(ycol) [lindex $c 1]
	    set cols "[$current(frame) get bin list]"
	    PopUp bindialog(xcol) $w.param.xcol.m $cols
	    PopUp bindialog(ycol) $w.param.ycol.m $cols

	    set bindialog(factor) [$current(frame) get bin factor]
	    set c [$current(frame) get bin cursor]
	    set bindialog(x) [lindex $c 0]
	    set bindialog(y) [lindex $c 1]
	    set bindialog(filter) [$current(frame) get bin filter]
	} else {
	    set bindialog(factor) {}
	    set bindialog(x) {}
	    set bindialog(y) {}
	    set bindialog(filter) {}
	}
    }
}

proc BinApplyDialog {} {
    global current
    global bin
    global bindialog

    if {$current(frame) != ""} {
	if {[$current(frame) has bin]} {
	    set bin(factor) $bindialog(factor)

	    SetWatchCursor
	    if {$bindialog(auto)} {
		$current(frame) bin to $bin(factor) about center \
		    \{$bindialog(xcol)\} \{$bindialog(ycol)\} \
		    \{$bindialog(filter)\}
	    } else {
		$current(frame) bin to $bin(factor) \
		    about $bindialog(x) $bindialog(y) physical \
		    \{$bindialog(xcol)\} \{$bindialog(ycol)\} \
		    \{$bindialog(filter)\}
	    }

 	    UpdateContourMenu
	    UpdateGraphXAxis
	    UnsetWatchCursor

	    # the bining center will shift
	    UpdateBinDialog
	}
    }

    # reset auto center
    set bindialog(auto) 0
}

proc BinDestroyDialog {} {
    global bindialog

    destroy $bindialog(top)
    destroy $bindialog(mb)
}

proc BinCutDialog {} {
    global bindialog

    set w [focus -displayof $bindialog(top)]

    if {![catch {set data [string range [$w get] [$w index sel.first] [expr {[$w index sel.last] - 1}]]}]} {
        clipboard clear -displayof $w
        clipboard append -displayof $w $data
        $w delete sel.first sel.last
    }
}

proc BinCopyDialog {} {
    global bindialog

    set w [focus -displayof $bindialog(top)]

    if {![catch {set data [string range [$w get] [$w index sel.first] [expr {[$w index sel.last] - 1}]]}]} {
        clipboard clear -displayof $w
        clipboard append -displayof $w $data
    }
}

proc BinPasteDialog {} {
    global tcl_platform
    global bindialog

    set w [focus -displayof $bindialog(top)]

    catch {
        if {"$tcl_platform(platform)" != "unix"} {
            catch {
                $w delete sel.first sel.last
            }
        }
        $w insert insert [selection get -displayof $w -selection CLIPBOARD]
        tkEntrySeeInsert $w
    }
}

proc BinClearDialog {} {
    global bindialog

    set w [focus -displayof $bindialog(top)]

    if {![catch {selection get -displayof $bindialog(top)}]} {
	$w delete sel.first sel.last
    }
}

proc UpdateBinMenu {} {
    global current
    global bin
    global buttons

    if {$current(frame) != ""} {
	if {[$current(frame) has fits]} {
	    if {[$current(frame) has bin]} {

		set bin(function) [$current(frame) get bin function]
		set bin(factor) [$current(frame) get bin factor]	    
		set bin(buffersize) [$current(frame) get bin buffer size]

		.menuBar entryconfig "Bin" -state normal
		$buttons(name).major.bin configure -state normal
	    } else {
		.menuBar entryconfig "Bin" -state disabled
		$buttons(name).major.bin configure -state disabled
	    }
	} else {
	    .menuBar entryconfig "Bin" -state normal
	    $buttons(name).major.bin configure -state normal
	}
    } else {
	.menuBar entryconfig "Bin" -state disabled
	$buttons(name).major.bin configure -state disabled
    }
}

proc ProcessBin {varName iName} {
    upvar $varName var
    upvar $iName i

    global bin

    while {$i < [llength $var]} {
	set item [lindex $var $i]
	switch -- $item {
	    about {
		BinAbout [lindex $var [expr $i+1]] [lindex $var [expr $i+2]]
		incr i 2
	    }
	    buffersize {
		incr i
		set bin(buffersize) [lindex $var $i]
		ChangeBinBufferSize
	    }
	    cols {
		BinCols [lindex $var [expr $i+1]] [lindex $var [expr $i+2]]
		incr i 2
	    }
	    factor {
		incr i
		set bin(factor) [lindex $var $i]
		ChangeBinFactor
	    }
	    filter {
		incr i
		BinFilter [lindex $var $i]
	    }
	    function {
		incr i
		set bin(function) [string tolower [lindex $var $i]]
		ChangeBinFunction
	    }
	    to {
		incr i \# eat the 'fit'
		BinToFit
	    }

	    default {incr i -1;break}
	}
	incr i
    }
}
