package provide DS9 1.0

proc BoxDialog {frame id} {
    global marker

    # see if we already have a header window visible

    set w ".marker$id"
    set mb ".mb$id"

    if [winfo exist $w] {
	return
    }

    # Init common items

    InitMarkerDialog $frame $id
    PropMenuMarkerDialog $frame $mb $id
    CoordMenuMarkerDialog $frame $mb $id CoordBoxCB

    # file menu items

    $mb.file add command -label "Apply" \
	-command "ApplyBoxDialog $frame $id"
    $mb.file add separator
    $mb.file add command -label "Close" \
	-command "CloseBoxDialog $frame $id"

    # box menu items

    set marker($frame,$id,sizeformat) pixels
    DistMenuMarkerDialog $frame $mb $id CoordBoxCB sizeformat
    $mb add cascade -label Size -menu $mb.sizeformat

    # Init box variables

    EditBoxCB $frame $id
    RotateBoxCB $frame $id

    # box specific callbacks

    $frame marker $id callback edit EditBoxCB $frame
    $frame marker $id callback edit MoveMarkerCB $frame
    $frame marker $id callback rotate RotateBoxCB $frame
    $frame marker $id callback delete DeleteBoxDialog $frame

    # Dialog

    CommonMarkerDialog $frame $id
    wm protocol $w WM_DELETE_WINDOW "CloseBoxDialog $frame $id"

    # Size

    label $w.ref.widthTitle -text "Width/Height"
    entry $w.ref.width -textvariable marker($frame,$id,width) \
	-width 13 -relief sunken -bd 2
    entry $w.ref.height -textvariable marker($frame,$id,height) \
	-width 13 -relief sunken -bd 2
    label $w.ref.sizeCoord -textvariable marker($frame,$id,sizeformat) \
	-relief groove -bd 2 -width 9 -padx 4 -text "pixels"
    label $w.ref.angleTitle -text "Angle"
    entry $w.ref.angle -textvariable marker($frame,$id,angle) \
	-width 13 -relief sunken -bd 2
    label $w.ref.angleUnit -relief groove -bd 2 -width 9 -padx 4 \
	-text "degrees"

    grid $w.ref.widthTitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.ref.width -row 1 -column 1 -padx 4 -sticky w
    grid $w.ref.height -row 1 -column 2 -padx 4 -sticky w
    grid $w.ref.sizeCoord -row 1 -column 3 -padx 4 -sticky w

    grid $w.ref.angleTitle -row 2 -column 0 -padx 4 -sticky w
    grid $w.ref.angle -row 2 -column 1 -padx 4 -sticky w
    grid $w.ref.angleUnit -row 2 -column 3 -padx 4 -sticky w

    # Buttons

    button $w.buttons.apply -text "Apply" \
	-command "ApplyBoxDialog $frame $id"
    button $w.buttons.close -text "Close" \
	-command "CloseBoxDialog $frame $id"
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w <Return> "ApplyBoxDialog $frame $id"
}

proc ApplyBoxDialog {frame id} {
    EditBoxDialog $frame $id
    RotateBoxDialog $frame $id
    ApplyMarkerDialog $frame $id
}

proc CloseBoxDialog {frame id} {
    global marker

    unset marker($frame,$id,width)
    unset marker($frame,$id,height)
    unset marker($frame,$id,angle)

    $frame marker $id delete callback edit EditBoxCB
    $frame marker $id delete callback edit MoveMarkerCB
    $frame marker $id delete callback rotate RotateBoxCB
    $frame marker $id delete callback delete DeleteBoxDialog
    DeleteMarkerCBs $frame $id

    DeleteMarkerDialog $frame $id
}

proc DeleteBoxDialog {frame id} {
    global marker

    unset marker($frame,$id,width)
    unset marker($frame,$id,height)
    unset marker($frame,$id,angle)
    unset marker($frame,$id,sizeformat)

    set mb ".mb$id"
    destroy $mb.sizeformat

    DeleteMarkerDialog $frame $id
}

proc EditBoxDialog {frame id} {
    global marker

    $frame marker $id box size \
	$marker($frame,$id,width) $marker($frame,$id,height) \
	$marker($frame,$id,sizeformat)
}

proc RotateBoxDialog {frame id} {
    global marker

    $frame marker $id box angle $marker($frame,$id,angle)
}

proc EditBoxCB {frame id} {
    global marker

    set f [$frame get marker $id box size $marker($frame,$id,sizeformat)]

    set marker($frame,$id,width) [lindex $f 0]
    set marker($frame,$id,height) [lindex $f 1]
}

proc RotateBoxCB {frame id} {
    global marker

    set marker($frame,$id,angle) [$frame get marker $id box angle]
}

proc CoordBoxCB {frame id} {
    MoveMarkerCB $frame $id
    EditBoxCB $frame $id
}

proc BoxDefaultDialog {} {
    global box
    global ed

    set w ".box"
    set title "Default Box"
    set ed(ok) 0
    set ed(width) $box(width)
    set ed(height) $box(height)

    DialogCreate $w $title -borderwidth 2
    frame $w.ed  -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.ed $w.buttons -fill x -ipadx 4 -ipady 4

    label $w.ed.title -text "Width"
    entry $w.ed.width -textvariable ed(width) -width 10 \
	-relief sunken -bd 2
    label $w.ed.unit -text "pixels" -relief groove -width 8
    
    label $w.ed.title2 -text "Height"
    entry $w.ed.height -textvariable ed(height) -width 10 \
	-relief sunken -bd 2
    label $w.ed.unit2 -text "pixels" -relief groove -width 8

    grid $w.ed.title -row 1 -column 0 -padx 4  -pady 1 -sticky w
    grid $w.ed.width -row 1 -column 1 -padx 4  -pady 1 -sticky w
    grid $w.ed.unit -row 1 -column 2 -padx 4  -pady 1 -sticky w
    grid $w.ed.title2 -row 2 -column 0 -padx 4  -pady 1 -sticky w
    grid $w.ed.height -row 2 -column 1 -padx 4  -pady 1 -sticky w
    grid $w.ed.unit2 -row 2 -column 2 -padx 4  -pady 1 -sticky w

    button $w.buttons.ok -text "OK" -default active \
	-command {set ed(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set ed(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set ed(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    $w.ed.width select range 0 end
    DialogWait $w ed(ok) $w.ed.width
    DialogDismiss $w

    if {$ed(ok)} {
	set box(width) $ed(width)
	set box(height) $ed(height)
    }
    unset ed(width)
    unset ed(height)
}
