package provide DS9 1.0

proc IISInit {} {
    global iis

    iis open $iis(ififo) $iis(ofifo) $iis(port) $iis(unix)
}

proc IISClose {} {
    iis close
}

# Callbacks

proc IISInitializeCmd {w h} {
    global iis

    # default frame size
    set iis(width) $w
    set iis(height) $h
}

proc IISInitFrameCmd {which} {
}

proc IISSetDisplayFrameCmd {which w h} {
    global iis

    IISGotoFrame $which
    IISLoadFrame $which
}

proc IISSetRefFrameCmd {which} {
}

proc IISEraseFrameCmd {which} {
    Frame$which iis erase
    IISResetTimer $which
}

proc IISMessageCmd {message} {
    global current
    $current(frame) iis message "\"$message\""
}

proc IISWritePixelsCmd {which ptr x y dx dy} {
    Frame$which iis set $ptr $x $y $dx $dy
    IISResetTimer $which
}

proc IISReadPixelsCmd {which ptr x y dx dy} {
    Frame$which get iis $ptr $x $y $dx $dy
    IISResetTimer $which
}

proc IISWCSCmd {which a b c d e f z1 z2 zt} {
    # if there is a change in config, we are not told until now
    IISLoadFrame $which

    Frame$which iis wcs $a $b $c $d $e $f $z1 $z2 $zt
}

proc IISSetCursorPosCmd {x y} {
    global current

    $current(frame) iis cursor $x $y image
}

proc IISGetCursorPosCmd {} {
    global current

    if {[$current(frame) has iis]} {
	set num [string range $current(frame) end end]
	return "[$current(frame) get iis cursor] $num"
    } else {
	# default to first frame
	return "1 1 0"
    }
}

proc IISCursorModeCmd {state} {
    global iis
    global canvas
    global current

    if {$state != $iis(state)} {
	set iis(state) $state
	$current(frame) iis cursor mode $state

	if {$state} {
	    # set key bindings
	    bind $canvas(name) <Any-Key> [list IISCursorKey %A %x %y]
	    bind $canvas(name) <f> {}
	    $canvas(name) bind $current(frame) <c> {}

	    if {$iis(abort)} {
		set iis(abort) 0
		set iis(cursor) 1
	    } else {
		set iis(cursor) 1
		IISCursorTimer
	    }
	} else {
	    # restore canvas key bindings
	    bind $canvas(name) <Any-Key> {}
	    bind $canvas(name) <f> {ToggleFreeze}
	    $canvas(name) bind $current(frame) <c> \
		[list DisplayCoordDialog $current(frame) %x %y]

	    set iis(abort) 1
	    set iis(cursor) 0
	}
    }
}

proc IISCursorTimer {} {
    global iis
    global canvas
    global ds9

    switch $iis(cursor) {
	0 {
	    set iis(cursor) 0
	    set iis(abort) 0
	    $canvas(name) configure -cursor {}
	}
	1 {
	    $canvas(name) configure -cursor circle
	    set iis(cursor) 2
	    after 1000 IISCursorTimer
	}
	2 {
	    $canvas(name) configure -cursor dot
	    set iis(cursor) 1
	    after 1000 IISCursorTimer
	}
    }
}

proc IISLoadFrame {which} {
    global iis

    if {(![Frame$which has iis]) || \
	([Frame$which get iis width] != $iis(width)) || \
	([Frame$which get iis height] != $iis(height))} {

	StartLoad
	Frame$which iis new $iis(width) $iis(height)
	FinishLoad
    }
}

proc IISGotoFrame {which} {
    global current

    if {$current(frame) != "Frame$which"} {
	CreateGotoFrame $which
    }
}

proc IISResetTimer {which} {
    global iis

    if {![info exists iis(timer$which)]} {
	after 500 IISTimer $which
    }
    set iis(timer$which) 1
}

proc IISTimer {which} {
    global iis

    if {$iis(timer$which)} {
	after 500 IISTimer $which
	set iis(timer$which) 0
    } else {
	Frame$which iis update
	unset iis(timer$which)
    }
}

proc IISCursorKey {key x y} {
    global canvas

    if {$key!={}} {
	set which [lindex [$canvas(name) gettags current] 0]
	if {$which != ""} {
	    if {[$which has iis]} {
		set num [string range $which end end]
		set coord [$which get image coordinates canvas $x $y]
		if {$coord != ""} {
		    $which iis cursor $x $y canvas
		    iis retcur [lindex $coord 0] [lindex $coord 1] $key $num
		}
	    } else {
		Error "This is not an IRAF Frame"
	    }
	}
    }
}
