package provide DS9 1.0

proc LineDialog {frame id} {
    global menu
    global marker
    global ds9

    # see if we already have a header window visible

    set w ".marker$id"
    set mb ".mb$id"

    if [winfo exist $w] {
	return
    }

    # init marker variables

    set marker($frame,$id,coord) physical
    set marker($frame,$id,format) degrees
    set marker($frame,$id,distformat) pixels
    AdjustCoordSystem marker($frame,$id,coord)

    # arrows

    set arrows [$frame get marker $id line arrow]
    set marker($frame,$id,p1arrow) [lindex $arrows 0]
    set marker($frame,$id,p2arrow) [lindex $arrows 1]

    EditLineCB $frame $id
    TextMarkerCB $frame $id
    ColorMarkerCB $frame $id
    PropertyMarkerCB $frame $id
    FontMarkerCB $frame $id

    # line specific callbacks

    $frame marker $id callback move EditLineCB $frame
    $frame marker $id callback edit EditLineCB $frame
    $frame marker $id callback text TextMarkerCB $frame
    $frame marker $id callback color ColorMarkerCB $frame
    $frame marker $id callback property PropertyMarkerCB $frame
    $frame marker $id callback font FontMarkerCB $frame
    $frame marker $id callback delete DeleteLineDialog $frame

    # define menus

    MBMarkerDialog $frame $mb
    FileMenuMarkerDialog $frame $mb $id
    ColorMenuMarkerDialog $frame $mb $id
    PropMenuLineDialog $frame $mb $id
    CoordMenuMarkerDialog $frame $mb $id EditLineCB
    FontMenuMarkerDialog $frame $mb $id
    DistMenuMarkerDialog $frame $mb $id DistCB distformat
    $mb add cascade -label Distance -menu $mb.distformat

    # file menu items

    $mb.file add command -label "Apply" \
	-command "ApplyLineDialog $frame $id"
    $mb.file add separator
    $mb.file add command -label "Close" \
	-command "CloseLineDialog $frame $id"

    # create window

    set type [string toupper "[$frame get marker $id type]"]
    toplevel $w -colormap $ds9(main)
    wm title $w $type
    wm iconname $w $type
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW "CloseLineDialog $frame $id"

    $w configure -menu $mb

    # Dialog

    frame $w.basic -relief groove -borderwidth 2
    frame $w.ref -relief groove -borderwidth 2
    frame $w.dist -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.basic $w.ref $w.dist $w.buttons -fill x -ipadx 4 -ipady 4

    # ID

    label $w.basic.idTitle -text "Id"
    label $w.basic.idValue -text "$id"

    # Text

    label $w.basic.textTitle -text "Text"
    entry $w.basic.textValue -textvariable marker($frame,$id,text) \
	-width 40 -relief sunken -bd 2
    label $w.basic.fill -width 9

    grid $w.basic.idTitle -row 0 -column 0 -padx 4 -sticky w
    grid $w.basic.idValue -row 0 -column 1 -padx 4 -sticky w

    grid $w.basic.textTitle -row 1 -column 0 -padx 4 -sticky w
    grid $w.basic.textValue -row 1 -column 1 -padx 4 -sticky w
    grid $w.basic.fill -row 1 -column 2 -padx 4 -sticky w

    # Points

    label $w.ref.title -text "Points"
    entry $w.ref.x -textvariable marker($frame,$id,x) \
	-width 13 -relief sunken -bd 2
    entry $w.ref.y -textvariable marker($frame,$id,y) \
	-width 13 -relief sunken -bd 2
    checkbutton $w.ref.p1arrow -variable marker($frame,$id,p1arrow) \
	-text "Arrow" -command "ArrowLineDialog $frame $id"
    label $w.ref.coord -relief groove -bd 2 -width 9 -padx 4 \
	-textvariable marker($frame,$id,coord)
    entry $w.ref.x2 -textvariable marker($frame,$id,x2) \
	-width 13 -relief sunken -bd 2
    entry $w.ref.y2 -textvariable marker($frame,$id,y2) \
	-width 13 -relief sunken -bd 2
    checkbutton $w.ref.p2arrow -variable marker($frame,$id,p2arrow) \
	-text "Arrow" -command "ArrowLineDialog $frame $id"

    grid $w.ref.title -row 1 -column 0 -padx 4 -sticky w
    grid $w.ref.x -row 1 -column 1 -padx 4 -sticky w
    grid $w.ref.y -row 1 -column 2 -padx 4 -sticky w
    grid $w.ref.p1arrow -row 1 -column 3 -padx 4 -sticky w
    grid $w.ref.coord -row 1 -column 4 -padx 4 -sticky w

    grid $w.ref.x2 -row 2 -column 1 -padx 4 -sticky w
    grid $w.ref.y2 -row 2 -column 2 -padx 4 -sticky w
    grid $w.ref.p2arrow -row 2 -column 3 -padx 4 -sticky w

    # Length

    label $w.dist.title -text "Length"
    label $w.dist.dist -relief groove -bd 2 -width 12 -padx 4 \
	-textvariable marker($frame,$id,dist)
    label $w.dist.format -relief groove -bd 2 -width 9 -padx 4 \
	-textvariable marker($frame,$id,distformat)
    label $w.dist.fill -width 23

    grid $w.dist.title -row 0  -column 0 -padx 4 -sticky w
    grid $w.dist.dist -row 0 -column 1 -padx 4 -sticky w
    grid $w.dist.format -row 0 -column 2 -padx 4 -sticky w
    grid $w.dist.fill -row 0 -column 3 -padx 4 -sticky w

    # Buttons

    button $w.buttons.apply -text "Apply" \
	-command "ApplyLineDialog $frame $id"
    button $w.buttons.close -text "Close" \
	-command "CloseLineDialog $frame $id"
    pack $w.buttons.apply $w.buttons.close -side left -padx 10 -expand true

    bind $w <Return> "ApplyLineDialog $frame $id"
}

proc PropMenuLineDialog {frame mb id} {
    global menu
    global marker

    menu $mb.properties -tearoff 0 -selectcolor $menu(selectcolor)
    $mb.properties add checkbutton -label "Can Edit" \
	-variable marker($frame,$id,edit) \
	-command "PropertyMarkerDialog $frame $id edit"
    $mb.properties add checkbutton -label "Can Move" \
	-variable marker($frame,$id,move) \
	-command "PropertyMarkerDialog $frame $id move"
    $mb.properties add checkbutton -label "Can Delete" \
	-variable marker($frame,$id,delete) \
	-command "PropertyMarkerDialog $frame $id delete"
    $mb.properties add checkbutton -label "Include/Exclude" \
	-variable marker($frame,$id,include) \
	-command "PropertyMarkerDialog $frame $id include"
}

proc ApplyLineDialog {frame id} {
    global marker

    EditLineDialog $frame $id
    $frame marker $id text \{$marker($frame,$id,text)\}

    UpdateMarkerMenu
}

proc CloseLineDialog {frame id} {
    global marker

    unset marker($frame,$id,x2)
    unset marker($frame,$id,y2)

    $frame marker $id delete callback edit EditLineCB
    $frame marker $id delete callback move EditLineCB
    $frame marker $id delete callback text TextMarkerCB
    $frame marker $id delete callback color ColorMarkerCB
    $frame marker $id delete callback property PropertyMarkerCB
    $frame marker $id delete callback font FontMarkerCB
    $frame marker $id delete callback delete DeleteLineDialog

    DeleteMarkerDialog $frame $id
}

proc DeleteLineDialog {frame id} {
    global marker

    unset marker($frame,$id,x2)
    unset marker($frame,$id,y2)

    set mb ".mb$id"
    destroy $mb.distformat

    DeleteMarkerDialog $frame $id
}

proc EditLineDialog {frame id} {
    global marker

    $frame marker $id line point \
	$marker($frame,$id,x) $marker($frame,$id,y) \
	$marker($frame,$id,x2) $marker($frame,$id,y2) $marker($frame,$id,coord)
}

proc EditLineCB {frame id} {
    global marker

    set c $marker($frame,$id,coord)
    if {$c == "fk4" | $c == "fk5" | $c == "icrs" | \
	    $c == "galactic" | $c == "ecliptic"} {
	set p [$frame get marker $id line point \
			$marker($frame,$id,coord) $marker($frame,$id,format)]

	set marker($frame,$id,x) [lindex $p 0]
	set marker($frame,$id,y) [lindex $p 1]
	set marker($frame,$id,x2) [lindex $p 3]
	set marker($frame,$id,y2) [lindex $p 4]
    } else {
	set p [$frame get marker $id line point $marker($frame,$id,coord)]

	set marker($frame,$id,x) [lindex $p 0]
	set marker($frame,$id,y) [lindex $p 1]
	set marker($frame,$id,x2) [lindex $p 2]
	set marker($frame,$id,y2) [lindex $p 3]
    }

    DistCB $frame $id
}

proc DistCB {frame id} {
    global marker

    set marker($frame,$id,dist) \
	[$frame get marker $id line distance $marker($frame,$id,distformat)]
}

proc ArrowLineDialog {frame id} {
    global marker
    
    $frame marker $id line arrow \
	$marker($frame,$id,p1arrow) $marker($frame,$id,p2arrow)
}
