package provide DS9 1.0

proc NSVRDialog {} {
    global nsvr
    global ds9
    global menu

    if [winfo exists $nsvr(top)] {
	return
    }

    set w $nsvr(top)
    set title "Name Resolution"

    # variables

    set nsvr(system) fk5

    # create the window

    toplevel $w -colormap $ds9(main)
    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW NSVRDestroyDialog

    # menu

    $w configure -menu $nsvr(mb)

    menu $nsvr(mb) -tearoff 0
    $nsvr(mb) add cascade -label File -menu $nsvr(mb).file
    $nsvr(mb) add cascade -label "Name Server" -menu $nsvr(mb).name
    $nsvr(mb) add cascade -label "Format" -menu $nsvr(mb).format

    menu $nsvr(mb).file -tearoff 0 -selectcolor $menu(selectcolor)
    $nsvr(mb).file add command -label "Resolve" -command NSVRApplyDialog
    $nsvr(mb).file add command -label "Cancel" -command NSVRCancelDialog
    $nsvr(mb).file add command -label "Clear" -command NSVRClearDialog
    $nsvr(mb).file add separator
    $nsvr(mb).file add command -label "Pan To" -command NSVRPanDialog
    $nsvr(mb).file add command -label "Crosshair To" \
	-command NSVRCrosshairDialog
    $nsvr(mb).file add separator
    $nsvr(mb).file add command -label "Close" -command NSVRDestroyDialog

    menu $nsvr(mb).name -tearoff 0 -selectcolor $menu(selectcolor)
    $nsvr(mb).name add radiobutton -label "NED@SAO" \
	-variable nsvr(server) -value ned-sao
    $nsvr(mb).name add radiobutton -label "NED@ESO" \
	-variable nsvr(server) -value ned-eso
    $nsvr(mb).name add radiobutton -label "SIMBAD@SAO" \
	-variable nsvr(server) -value simbad-sao
    $nsvr(mb).name add radiobutton -label "SIMBAD@ESO" \
	-variable nsvr(server) -value simbad-eso

    menu $nsvr(mb).format -tearoff 0 -selectcolor $menu(selectcolor)
    $nsvr(mb).format add radiobutton -label "Degrees" \
	-variable nsvr(format) -value degrees
    $nsvr(mb).format add radiobutton -label "Sexagesimal" \
	-variable nsvr(format) -value sexagesimal

    frame $w.param -relief groove -borderwidth 2
    frame $w.status -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.param $w.status $w.buttons -fill x -expand true -ipadx 4 -ipady 4

    label $w.param.nametitle -text "Name"
    entry $w.param.name -textvariable nsvr(name) -width 32 -relief sunken -bd 2

    label $w.param.xtitle -text "a" -font symbol -width 1
    label $w.param.x -textvariable nsvr(x) -width 14 -relief groove -bd 2
    label $w.param.ytitle -text "d" -font symbol -width 1
    label $w.param.y -textvariable nsvr(y) -width 14 -relief groove -bd 2
    label $w.param.system -textvariable nsvr(system) \
	-width 10 -relief groove -bd 2

    grid rowconfigure $w.param 0 -pad 4
    grid rowconfigure $w.param 1 -pad 4
    grid rowconfigure $w.param 2 -pad 4

    grid $w.param.nametitle -row 0 -column 0 -padx 4 -pady 1 -sticky w
    grid $w.param.name -row 0 -column 1 -columnspan 4 -padx 4 -pady 1 -sticky w

    grid $w.param.xtitle -row 1 -column 0 -padx 4 -pady 1 -sticky w
    grid $w.param.x -row 1 -column 1 -padx 4 -sticky w
    grid $w.param.ytitle -row 1 -column 2 -padx 4 -pady 1 -sticky w
    grid $w.param.y -row 1 -column 3 -padx 4 -sticky w
    grid $w.param.system -row 1 -column 4 -padx 4 -pady 1 -sticky w

    label $w.status.item -textvariable nsvr(status)
    pack $w.status.item -anchor w -pady 4

    set nsvr(apply) \
	[button $w.buttons.apply -text "Resolve" -command NSVRApplyDialog]
    set nsvr(cancel) \
	[button $w.buttons.cancel -text "Cancel" -command NSVRCancelDialog \
	     -state disabled]

    button $w.buttons.close -text "Close" -command NSVRDestroyDialog
    pack $w.buttons.apply $w.buttons.cancel $w.buttons.close \
	-side left -padx 10 -expand true

    NSVRStatus {}
}

proc NSVRDestroyDialog {} {
    global nsvr

    if {[info exists nsvr(token)]} {
	http::reset $nsvr(token)
    }
    destroy $nsvr(top)
    destroy $nsvr(mb)
}

proc NSVRApplyDialog {} {
    global nsvr

    NSVRStatus {}
    $nsvr(apply) configure -state disabled
    $nsvr(cancel) configure -state normal

    if {$nsvr(name)!=""} {
	NSVRStatus "Looking up $nsvr(name)"
	set nsvr(x) {}
	set nsvr(y) {}
	set data [NameSVR $nsvr(server) $nsvr(name) $nsvr(format)]
	set nsvr(x) [lindex $data 0]
	set nsvr(y) [lindex $data 1]

	if {($nsvr(x) == "") || ($nsvr(y) == "")} {
	    set nsvr(x) {}
	    set nsvr(y) {}
	    NSVRStatus "$nsvr(name) not found"

	    $nsvr(apply) configure -state normal
	    $nsvr(cancel) configure -state disabled
	    return
	}
    }

    NSVRStatus {Done}
    $nsvr(apply) configure -state normal
    $nsvr(cancel) configure -state disabled
}

proc NSVRCancelDialog {} {
    global nsvr

    if {[info exists nsvr(token)]} {
	http::reset $nsvr(token)
    }

    $nsvr(apply) configure -state normal
    $nsvr(cancel) configure -state disabled
}

proc NSVRStatus {message} {
    global nsvr

    set nsvr(status) "Status: $message"
}

proc NSVRClearDialog {} {
    global nsvr

    set nsvr(name) ""
    set nsvr(x) ""
    set nsvr(y) ""
    NSVRStatus {}
}

proc NSVRPanDialog {} {
    global nsvr

    if {$nsvr(x) != ""} {
	PanTo $nsvr(x) $nsvr(y) $nsvr(system)
    }
}

proc NSVRCrosshairDialog {} {
    global nsvr
    global current
    global ds9

    if {($current(frame) != "") && ($nsvr(x) != "")} {
	set ds9(mode) crosshair
	ChangeMode
	$current(frame) crosshair $nsvr(x) $nsvr(y) $nsvr(system)
    }
}

proc NameSVR {where which format} {
    global http
    
    switch $where {
	ned-sao {return [NEDSAO $which $format]}
	ned-eso {return [NEDESO $which $format]}
	simbad-sao {return [SIMBADSAO $which $format]}
	simbad-eso {return [SIMBADESO $which $format]}
    }
}

proc NEDSAO {which format} {
    set token [http::geturl http://www-cfa.harvard.edu/catalog/ned?$which]
    set data [http::data $token]

    if {$data != "\n"} {
	switch $format {
	    degrees {return \
		"[expr h2d([Sex2D [lindex $data 0]])] [Sex2D [lindex $data 1]]"}
	    sexagesimal {return $data}
	}
    }
    return {}
}

proc NEDESO {which format} {
    set token [http::geturl http://archive.eso.org/skycat/servers/ned-server?&o=$which]
    set data [http::data $token]

    if {$data != ""} {
	set r {}
	set d {}
	scan $data "Id              ra              dec\n-----------      ---------      ----------\n %s %s %f %f" cat num r d
	if {($r != "") && ($d != "")} {
	    switch $format {
		degrees {return "$r $d"}
		sexagesimal {return \
		    "[format %.3@ [expr d2h($r)]] [format %.3@ $d]"}
	    }
	}
    }
    return {}
}

proc SIMBADSAO {which format} {
    set token \
	[http::geturl http://www-cfa.harvard.edu/catalog/simbad-cfa?$which]
    set data [http::data $token]

    if {$data != ""} {
	if {([string range $data 0 2] == "row")} {
	    return {}
	} else {
	    switch $format {
		degrees {return \
		    "[expr h2d([Sex2D [lindex $data 0]])] [Sex2D [lindex $data 1]]"}
		sexagesimal {return $data}
	    }
	}
    }
    return {}
}

proc SIMBADESO {which format} {
    set token [http::geturl http://archive.eso.org/skycat/servers/sim-server?&o=$which]
    set data [http::data $token]

    if {$data != ""} {
	set r {}
	set d {}
	scan $data "Id              ra              dec\n-----------      ---------      ----------\n %s %s %f %f" cat num r d
	if {($r != "") && ($d != "")} {
	    switch $format {
		degrees {return "$r $d"}
		sexagesimal {return \
		    "[format %.3@ [expr d2h($r)]] [format %.3@ $d]"}
	    }
	}
    }
    return {}
}


