package provide DS9 1.0

proc ChangeColorScale {} {
    global current

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) colorscale $current(scale)
	UnsetWatchCursor
    }
}

proc ScaleScope {} {
    global current
    global scale

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) clip scope $scale(scope)
	UpdateGraphYAxis
	UnsetWatchCursor
    }
}

proc ScaleMode {} {
    global current
    global scale

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) clip mode $scale(mode)
	UpdateGraphYAxis
	UnsetWatchCursor
    }
}

proc MinMaxMode {} {
    global current
    global minmax

    if {$current(frame) != ""} {
	SetWatchCursor
	$current(frame) clip minmax mode $minmax(mode)
	UpdateGraphYAxis
	UnsetWatchCursor
    }
}

proc UpdateScaleMenu {} {
    global current
    global scale
    global minmax
    global buttons

    if {$current(frame) != ""} {
	if {![$current(frame) has iis]} {
	    .menuBar entryconfig "Scale" -state normal
	    $buttons(name).major.scale configure -state normal

	    set count [$current(frame) get fits count]

	    if {$count == 0} {
		.menuBar.scale.minmax entryconfig "DATAMIN DATAMAX" \
		    -state normal
		.menuBar.scale.minmax entryconfig "IRAF-MIN IRAF-MAX" \
		    -state normal
	    } else {
		if [$current(frame) has datamin] {
		    .menuBar.scale.minmax entryconfig "DATAMIN DATAMAX" \
			-state normal
		} else {
		    .menuBar.scale.minmax entryconfig "DATAMIN DATAMAX" \
			-state disabled
		}
		if [$current(frame) has irafmin] {
		    .menuBar.scale.minmax entryconfig "IRAF-MIN IRAF-MAX" \
			-state normal
		} else {
		    .menuBar.scale.minmax entryconfig "IRAF-MIN IRAF-MAX" \
			-state disabled
		}
	    }

	    set current(scale) [$current(frame) get colorscale]
	    set scale(scope) [$current(frame) get clip scope]
	    set scale(mode) [$current(frame) get clip mode]
	    set minmax(mode) [$current(frame) get clip minmax mode]

	    return
	}
    }

    # else disable the menu

    .menuBar entryconfig "Scale" -state disabled
    $buttons(name).major.scale configure -state disabled
    
    set current(scale) linear
    set scale(scope) local
    set scale(mode) minmax
    set minmax(mode) scan
}

proc ZScaleDialog {} {
    global zscale
    global current

    set w ".zscale"
    set zscale(ok) 0
    set length 300

    set params [$current(frame) get clip zscale param]
    set zscale(contrast) [lindex $params 0]
    set zscale(size) [lindex $params 1]
    set zscale(line) [lindex $params 2]

    DialogCreate $w "ZScale Parameters" -borderwidth 2
    frame $w.zscale  -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.zscale $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    scale $w.zscale.contrast -from 0 -to 1 -length $length \
	-variable zscale(contrast) -orient horizontal -label "Contrast" \
	-tickinterval .2 -showvalue true -resolution .05
    scale $w.zscale.size -from 0 -to 1000 -length $length \
	-variable zscale(size) -orient horizontal -label "Number of Samples" \
	-tickinterval 200 -showvalue true -resolution 10
    scale $w.zscale.line -from 0 -to 500 -length $length \
	-variable zscale(line) -orient horizontal -label "Samples per Line" \
	-tickinterval 100 -showvalue true -resolution 5
    pack $w.zscale.contrast $w.zscale.size $w.zscale.line -side top

    button $w.buttons.ok -text "OK" -default active -command {set zscale(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set zscale(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set zscale(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w zscale(ok)
    DialogDismiss $w

    if {$zscale(ok)} {
	SetWatchCursor
	if {$zscale(line) == 0} {
	    set zscale(line) 1
	}
	$current(frame) clip zscale param \
	    $zscale(contrast) $zscale(size) $zscale(line)
	UpdateGraphYAxis
	UnsetWatchCursor
    }

    # we want to destroy this window

    destroy $w 
}

proc SampleDialog {} {
    global sample
    global current

    set w ".sample"
    set sample(ok) 0
    set length 300

    set sample(incr) [$current(frame) get clip minmax param]

    DialogCreate $w "Sample Parameters" -borderwidth 2
    frame $w.sample  -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.sample $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    scale $w.sample.incr -from 0 -to 1000 -length $length \
	-variable sample(incr) -orient horizontal -label "Sample Increment" \
	-tickinterval 200 -showvalue true
    pack $w.sample.incr

    button $w.buttons.ok -text "OK" -default active -command {set sample(ok) 1}
    button $w.buttons.cancel -text "Cancel" -command {set sample(ok) 0}
    pack $w.buttons.ok -side left -padx 10
    pack $w.buttons.cancel -side right -padx 10

    bind $w <Return> {set sample(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"

    DialogCenter $w 
    DialogWait $w sample(ok)
    DialogDismiss $w

    if {$sample(ok)} {
	SetWatchCursor
	if {$sample(incr) == 0} {
	    set sample(incr) 1
	}
	$current(frame) clip minmax param $sample(incr)
	UpdateGraphYAxis
	UnsetWatchCursor
    }

    # we want to destroy this window

    destroy $w 
}

proc ScaleLimitsDialog {} {
    global current
    global graph
    global menu
    global lim

    set w ".lim"
    set lim(ok) 0

    set limits [$current(frame) get clip]
    set lim(low) [lindex $limits 0]
    set lim(high) [lindex $limits 1]

    DialogCreate $w "Scale Limits" -borderwidth 2
    set lim(scale) linear

    frame $w.top  -relief groove -borderwidth 2
    frame $w.buttons -relief groove -borderwidth 2
    pack $w.top $w.buttons -ipadx 4 -ipady 4 -fill x -expand true

    frame $w.top.graph
    frame $w.top.graph.hist
    frame $w.top.graph.scale
    pack $w.top.graph
    pack $w.top.graph.hist $w.top.graph.scale -side left

    # graph
    
    set bgcolor [$w.top cget -background]
    set hist [blt::barchart $w.top.graph.hist.chart \
		  -width 400 -height 200 -title \
		  "Pixel Distribution" -plotrelief groove \
		  -plotbackground $bgcolor -plotpadx 0 -plotpady 0]

    set fgcolor [$hist cget -foreground]
    $hist legend configure -hide yes
    $hist element create bar1 -xdata histX -ydata histY -hide yes \
	-foreground $fgcolor -background $fgcolor -relief flat
    $hist xaxis configure -title "" -hide yes \
	-color $fgcolor -tickfont $graph(font) -ticklength 4 
    $hist yaxis configure -title "" -hide yes

    button $w.top.graph.hist.b -text "Show Histogram" \
	-command "UpdateHist $hist"
    pack $w.top.graph.hist.b $hist -side top

    # Scale

    radiobutton $w.top.graph.scale.linear -text "Linear" -variable lim(scale) \
	-value linear  -selectcolor $menu(selectcolor) \
	-command "$hist yaxis configure -logscale 0"
    radiobutton $w.top.graph.scale.log -text "Log" -variable lim(scale) \
	-value log  -selectcolor $menu(selectcolor) \
	-command "$hist yaxis configure -logscale 1 -min 1"
    pack $w.top.graph.scale.linear \
	$w.top.graph.scale.log \
	-side top -anchor w

    pack $w.top.graph.scale -side left

    # Cut Levels

    frame $w.top.low
    frame $w.top.high
    pack $w.top.low $w.top.high -side left -padx 10

    label $w.top.low.title -text "Low"
    entry $w.top.low.value -textvariable lim(low) -width 9 -relief sunken -bd 2
    pack $w.top.low.title $w.top.low.value -side left -padx 4

    label $w.top.high.title -text "High"
    entry $w.top.high.value -textvariable lim(high) -width 9 -relief sunken \
	-bd 2
    pack $w.top.high.title $w.top.high.value -side left -padx 4

    # buttons

    button $w.buttons.ok -text "OK" -default active -command {set lim(ok) 1}
    button $w.buttons.apply -text "Apply" \
	-command {ScaleLimit $lim(low) $lim(high)}
    button $w.buttons.cancel -text "Cancel" -command {set lim(ok) 0}
    pack $w.buttons.ok $w.buttons.apply $w.buttons.cancel \
	-side left -padx 10 -expand true

    bind $w <Return> {set lim(ok) 1}
    bind $w <Alt-o> "tkButtonInvoke $w.buttons.ok"
    bind $w <Alt-c> "tkButtonInvoke $w.buttons.cancel"
    bind $w <Alt-a> "tkButtonInvoke $w.buttons.apply"

    DialogCenter $w 
    DialogWait $w lim(ok)
    DialogDismiss $w

    if {$lim(ok)} {
	global user
	set user(low) $lim(low)
	set user(high) $lim(high)
	ScaleLimit $lim(low) $lim(high)
    }

    # we want to destroy this window

    destroy $w 
}

proc UpdateHist {hist} {
    global current
    global graph
    global histX histY

    set width [$current(frame) get histogram histX histY]

    # add an offset for log scaling if y min is zero

    if {$histY(min) == 0} {
	histY set [histY + .001]
    }

    $hist element configure bar1 -xdata histX -ydata histY \
	-hide no -barwidth $width
    $hist xaxis configure -hide no \
	-min [expr $histX(min)-$width] -max [expr $histX(max)+$width]
    $hist yaxis configure -min 1
}

proc ScaleLimit {low high} {
    global current
    global scale
    global user

    if {$current(frame) != ""} {
	SetWatchCursor
	set scale(mode) user
	set user(low) $low
	set user(high) $high
	$current(frame) clip user $low $high
	$current(frame) clip mode $scale(mode)
	UpdateGraphYAxis
	UnsetWatchCursor
    }
}

proc ScaleLimitLow {low} {
    global current
    global scale
    global user

    if {$current(frame) != ""} {
	SetWatchCursor
	set scale(mode) user
	set user(low) $low
	$current(frame) clip user low $low
	$current(frame) clip mode $scale(mode)
	UpdateGraphYAxis
	UnsetWatchCursor
    }
}

proc ScaleLimitHigh {high} {
    global current
    global scale
    global user

    if {$current(frame) != ""} {
	SetWatchCursor
	set scale(mode) user
	set user(high) $high
	$current(frame) clip user high $high
	$current(frame) clip mode $scale(mode)
	UpdateGraphYAxis
	UnsetWatchCursor
    }
}

