%option caseless
%array

%{

#include <stdio.h>
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <file.h>
#include <macro.h>
#include <strtod.h>
#include <filter.h>

#define MAX_INCLUDE_DEPTH 100
#define SEGINC 1024

#define LARGE_NUMBER 65536

/* must match regions.h */
#define PSTOP	-142857.142857

/* stolen from fitsy/xos.h */
#define X__PI	3.14159265358979323846
#define X_2PI	( 2 * X__PI )
#define X_R2D	(X_2PI / 360.0)
#define X_R2H	(X_2PI /  24.0)
#define X_H2D	(360.0 /  24.0)

#define r2h(r)	( (r) / X_R2H )
#define h2r(d)	( (d) * X_R2H )
#define r2d(r)	( (r) / X_R2D )
#define d2r(d)	( (d) * X_R2D )
#define h2d(r)	( (r) * X_H2D )
#define d2h(d)	( (d) / X_H2D )

/* region types */
#define TY_ANNULUS  1
#define TY_BOX      2
#define TY_CIRCLE   3
#define TY_ELLIPSE  4
#define TY_FIELD    5
#define TY_LINE     6
#define TY_PANDA    7
#define TY_PIE      8
#define TY_POINT    9
#define TY_POLYGON 10

/* oft-used checks on argument and coord types */
#define XARG    ((narg % 2) == 1)
#define YARG    ((narg % 2) == 0)
#define POSARG  ((regid == TY_POLYGON) || (narg<=2))
#define ANGARG (((regid == TY_BOX)     && (narg==5)) || \
                ((regid == TY_ELLIPSE) && (narg==5)) || \
                ((regid == TY_PANDA)   && (narg==3)) || \
                ((regid == TY_PANDA)   && (narg==4)) || \
                ((regid == TY_PIE)     && (narg==3)) || \
                ((regid == TY_PIE)     && (narg==4)) )

#define PHYSCOORDS (filt->fhd->table && \
                   (filt->type == TYPE_EVENTS) && !filt->evsect)

#define NOWCSSIZE (nowcs(wcs) || (cdelt1==0) || (cdelt2==0))

/* format statements */
#define DFMT1 ",%f"
#define DFMT2  ",%f,%f"

static YY_BUFFER_STATE include_stack[MAX_INCLUDE_DEPTH];
static int include_stack_ptr = 0;
static int parse_error = 0;

static int colargs;
static int extranew;
static int filtlen;
static int initbindefs;
static int regtype;
static int regid;
static int laststart;
static int maxfiltseg;
static int narg;
static int nparen;
static int nsparen;
static int nrparen;
static int nfiltseg;
static int nrange;
static int nroutine;
static int nshape;
static int nregion;
static int segid;
static char *filter;
static char *rangename;
static Filter filt;

static char regname[SZ_LINE];
static char colnames[2][SZ_LINE];
static char wcsnames[2][SZ_LINE];
static char macrobuf[SZ_LINE];
static char *rnames[] = {"annulus", "box", "circle", "ellipse", 
                         "line", "panda", "pie", "point",
			 "polygon", "field", NULL};
static int rargs[] =     {4,5,3,5,4,8,4,2,-1,0,0};
struct WorldCoor *wcs=NULL;
static FiltSeg *filtseg=NULL;

static char sbuf[SZ_LINE];
static char *fcoordsys=NULL;
static char *gcoordsys=NULL;
static char *tstr;
static double xpos, ypos;
static double xpix, ypix;
static double cdelt1, cdelt2;
static double tlmin[2];
static double tltyp[2];
static double dval, dval2;
static int offscl;
static int xtype, ytype;

static int _FiltLexTypeCmp(const void *s1, const void *s2);
static int _FiltLexTypeMatch(int ftype, int ttype);
static int _FiltLexAccel _PRx((int n));
static int _FiltLexRegionPanda _PRx((void));
static int _FiltLexLcx _PRx((void));
static void _FiltLexOp _PRx((char *s));
static void _FiltLexTypeSet _PRx((int type));
static void _FiltLexNew _PRx((int flag));
static void _FiltLexVarArgs _PRx((void));
static void _FiltLexCat _PRx((char *s));
static void _FiltLexRegionStart _PRx((int docols));
static void _FiltLexRegionEnd _PRx((int flag));
static void _FiltLexRangeAdd _PRx((char *xname, char *str, int type, int n));
static void _FiltLexArgCheck _PRx((char *name, int args));
static void _FiltLexSetWcs _PRx((char *s));
static void _FiltLexWcsCols _PRx((FITSHead header, char *name1, char *name2));
static void _FiltLexNum _PRx((int lcx));
static void _FiltLexEnd _PRx((void));
static void _FiltLexMake _PRx((void));
static char *_FiltLexGetWcs _PRx((void));
static char *_FiltLexBase _PRx((double *d, int nd, int *type, int *op, int f));

%}

DIG	[0-9]
INT	[-+]?{DIG}+
FLOAT1  [-+]?{DIG}+\.?([eE][-+]?{DIG}+)?
FLOAT2  [-+]?{DIG}*\.{DIG}+([eE][-+]?{DIG}+)?
FLOAT   ({FLOAT1}|{FLOAT2})
NUM	({INT}|{FLOAT})
HMS	{INT}{SP}[:hH]{SP}{INT}{SP}[:mM]{SP}{NUM}{SP}[s]?
DMS	{INT}{SP}[:dD]{SP}{INT}{SP}[:mM]{SP}{NUM}{SP}[s]?
WCS	{HMS}|{DMS}
DEG	{NUM}d
RAD	{NUM}r
IPIX	{NUM}i
PPIX	{NUM}p
ASEC	{NUM}\"
AMIN	{NUM}'
COORSYS	(j2000|fk5|icrs|b1950|fk4|galactic|ecliptic|linear|image|physical|amplifier|detector)
NAME	[A-Za-z~_][0-9A-Za-z~_]*(\[.*\])?
FILE	[A-Za-z~_/\.][0-9A-Za-z~_/\-\.]*(\[.*\])?
REGION	[\!+-]?(ann|annulus|box|cir|circle|ell|ellipse|lin|line|pan|panda|pie|poi|point|pol|polygon|fie|field)
OP	(&&|\|\||==|!=|<=|>=|<<|>>|[&\|=<>+\-*/%^])
STR	(\"[^\"]*\"|'[^']*')
SP	[ \t]*
EOL	([;\n]|\\n)
SEP	(,|{EOL})
COM	#
US	(unsigned{SP})?
II	(int{SP})?
DTYPE	(\({US}char\)|\({US}short{II}\)|\({US}int\)|\({US}long{II}\)|\(float\)|\(double\))

%s RLIST
%s RTINE
%s REG
%x AREG
%x INCLUDE
%x COMM

%%

{DTYPE}	    {_FiltLexCat(yytext);}

{COORSYS}   { _FiltLexSetWcs(yytext); }

global      { BEGIN COMM; }
{COM}       { if( YY_START == REG ) _FiltLexRegionEnd(1); BEGIN COMM; }
<COMM>{EOL} { BEGIN INITIAL; }
<COMM>.     { /* ignore comments up to eol */ ; }

[+-]?text\(.*\)	 { ; /* ignore text markers */ }
[+-]?ruler\(.*\) { ; /* ignore ruler markers */ }

\(/{NAME},{NAME}\){SP}={SP}{REGION} {
		if( YY_START == REG ) _FiltLexRegionEnd(1);
		/* Found an asc-style region */
		colargs = 0;
		laststart = YY_START;
		BEGIN AREG;
}
<AREG>{NAME}	{
		if( colargs > 2 )
		  _filterror("illegal Chandra region (too many column args)");
		if( !(tstr=FilterSymbolEnter(filt, yytext)) )
		  _filterror("missing column or header parameter");
		strcpy(colnames[colargs],tstr);
		colargs++;
}
<AREG>,		{ ; }
<AREG>\(	{ ; }
<AREG>\)	{ ; }
<AREG>= 	{	
		/* end of asc part of region */
		if( colargs != 2 )
		  _filterror("illegal Chandra region (wrong number of columns)");
		BEGIN INITIAL;
}

[+-]?field([ \t]+|{SP}\({SP})?             |
{REGION}([ \t]+|{SP}\({SP})/{NUM}  {
		/* finish off previous */
		if( YY_START == REG ) _FiltLexRegionEnd(1);
		/* Found a region */
		BEGIN REG;
                if( filt->type == TYPE_EVENTS ){
		  /* if asc-style, re-do wcs for specified columns */
		  if( colargs == 2 ){
		    _FiltLexRegionStart(2);
                    _FiltLexWcsCols(filt->fhd, colnames[0], colnames[1]);
                  }
		  /* set up default columns */
                  else{
		    _FiltLexRegionStart(1);
                  }
                }
                /* its an image */
                else{
		  _FiltLexRegionStart(0);
                }
}
{REGION}([ \t]+|{SP}\({SP})/{NAME}  {
                /* can't happen */
		_filterror("invalid characters following the region name");
}
<REG>{NUM}	{
		_FiltLexVarArgs();
                /* process this pure number in the current coord system */
                _FiltLexNum(_FiltLexLcx());
}
<REG>{IPIX}	{
		_FiltLexVarArgs();
                /* remove trailing unit character */
                yytext[strlen(yytext)-1] = '\0';
                /* process this  as a pure number in image coord sys */
                _FiltLexNum(LCX_IMAGE);
}
<REG>{PPIX}	{
		_FiltLexVarArgs();
                /* remove trailing unit character */
                yytext[strlen(yytext)-1] = '\0';
                /* process this  as a pure number in physical coord sys */
                _FiltLexNum(LCX_PHYS);
}
<REG>{WCS}	{
		char *coordsys;
		if( nowcs(wcs) )
		  _filterror("no WCS information available in file");
		_FiltLexVarArgs();
		/* save x value for processing with y next time */
		if( XARG ){
		  xpos = SAOstrtod(yytext,NULL);
		  xtype = SAOdtype;
		  if( (xtype !=0) && (xtype != '.') && (xtype != 'd') ){
		    coordsys = _FiltLexGetWcs();
		    /* arg1 coords are hms, but ecliptic, galactic are deg */
		    if( !coordsys                             ||
		        (strncasecmp(coordsys, "ecl", 3)     &&
			 strncasecmp(coordsys, "gal", 3))    ){
		      xpos = h2d(xpos);
		    }
		  }
		}
		else{
		  ypos = SAOstrtod(yytext,NULL);
		  ytype = SAOdtype;
                  /* convert wcs to image pixels */
		  wcsc2pix(wcs, xpos,ypos, _FiltLexGetWcs(), &xpix,&ypix,
			   &offscl);
                  /* convert to physical for single event filtering */
                  if( PHYSCOORDS ){
		    xpix = tli2p(xpix,tlmin[0],tltyp[0]);
		    ypix = tli2p(ypix,tlmin[1],tltyp[1]);
                  }
		  sprintf(sbuf, DFMT2, xpix, ypix);
		  _FiltLexCat(sbuf); 
		}
}
<REG>{DEG}	{
		if( nowcs(wcs) )
		  _filterror("no WCS information available in file");
		_FiltLexVarArgs();
		/* handle position arguments */
		if( POSARG ){
		  /* save x value for processing with y next time */
		  if( XARG ){
		    xpos = strtod(yytext,NULL);
		  }
		  else{
		    ypos = strtod(yytext,NULL);
                    /* convert wcs to image pixels */
		    wcsc2pix(wcs, xpos, ypos, _FiltLexGetWcs(),
				  &xpix, &ypix, &offscl);
                    /* convert to physical for single event filtering */
                    if( PHYSCOORDS ){
	  	      xpix = tli2p(xpix,tlmin[0],tltyp[0]);
		      ypix = tli2p(ypix,tlmin[1],tltyp[1]);
                    }
		    sprintf(sbuf, DFMT2, xpix, ypix);
		    _FiltLexCat(sbuf); 
		  }
		}
                /* angle arguments are just passed along */
		else if( ANGARG ){
		  dval = strtod(yytext,NULL);
		  sprintf(sbuf, DFMT1, dval);
		  _FiltLexCat(sbuf); 
                }
		/* handle size arguments */
		else{
		  if( NOWCSSIZE )
		    _filterror("no WCS information available in file");
		  /* arg 1 is ra, arg2 is dec */
		  if( XARG )
		    dval = ABS(strtod(yytext,NULL)/cdelt1);
		  else
		    dval = ABS(strtod(yytext,NULL)/cdelt2);
		  sprintf(sbuf, DFMT1, dval);
		  _FiltLexCat(sbuf); 
		}
}
<REG>{RAD}	{
		if( nowcs(wcs) )
		  _filterror("no WCS information available in file");
		_FiltLexVarArgs();
		/* handle position arguments */
		if( POSARG ){
		  /* save x value for processing with y next time */
		  if( XARG ){
		    xpos = r2d(strtod(yytext,NULL));
		  }
		  else{
		    ypos = r2d(strtod(yytext,NULL));
                    /* convert wcs to image pixels */
		    wcsc2pix(wcs, xpos, ypos, _FiltLexGetWcs(),
				  &xpix, &ypix, &offscl);
                    /* convert to physical for single event filtering */
                    if( PHYSCOORDS ){
	  	      xpix = tli2p(xpix,tlmin[0],tltyp[0]);
		      ypix = tli2p(ypix,tlmin[1],tltyp[1]);
                    }
		    sprintf(sbuf, DFMT2, xpix, ypix);
		    _FiltLexCat(sbuf); 
		  }
		}
                /* angle arguments are just passed along */
		else if( ANGARG ){
		  dval = r2d(strtod(yytext,NULL));
		  sprintf(sbuf, DFMT1, dval);
		  _FiltLexCat(sbuf); 
                }
		/* handle size arguments */
		else{
		  if( NOWCSSIZE )
		    _filterror("no WCS information available in file");
		  /* arg 1 is ra, arg2 is dec */
		  if( XARG )
		    dval = r2d(ABS(strtod(yytext,NULL)/cdelt1));
		  else
		    dval = r2d(ABS(strtod(yytext,NULL)/cdelt2));
		  sprintf(sbuf, DFMT1, dval);
		  _FiltLexCat(sbuf); 
		}
}
<REG>{AMIN}	{
		if( NOWCSSIZE )
		  _filterror("no WCS information available in file");
		_FiltLexVarArgs();
		/* arg 1 is ra, arg2 is dec */
		if( XARG )
		  dval = ABS(strtod(yytext,NULL)/(cdelt1*60.0));
		else
		  dval = ABS(strtod(yytext,NULL)/(cdelt2*60.0));
		sprintf(sbuf, DFMT1, dval);
		_FiltLexCat(sbuf); 
}
<REG>{ASEC}	{
		if( NOWCSSIZE )
		  _filterror("no WCS information available in file");
		_FiltLexVarArgs();
		/* arg 1 is ra, arg2 is dec */
		if( XARG )
		  dval = ABS(strtod(yytext,NULL)/(cdelt1*3600.0));
		else
		  dval = ABS(strtod(yytext,NULL)/(cdelt2*3600.0));
		sprintf(sbuf, DFMT1, dval);
		_FiltLexCat(sbuf); 
}
<REG>[A-z]{SP}={SP}{NUM}{SP}\)? {
		int n;
		if( (regid != TY_ANNULUS) && (regid != TY_PIE) ){
		  _filterror("'annulus' and 'pie' accelerators only - ");
		}
		n = strtol(strchr(yytext, '=')+1, NULL, 10);
		if( n <=0 ){
		  _filterror("invalid region accelerator");
		}
		else{
		  if( !_FiltLexAccel(n) ){
		    _filterror("invalid region accelerator");
		  }
		  BEGIN INITIAL;
		}
}
<REG>({NUM}{OP}|@{FILE}) {
		/* new expression without an expression separator ... sigh */
		_FiltLexRegionEnd(1);
		BEGIN INITIAL;
		yyless(0);
}
<REG>[,\(]  	{ 
		nsparen++;
}
<REG>[\)]	{ 
		nsparen--;
		/* less than 0 means it was not a region paren, we need it */
		if( nsparen <0 )
		  _FiltLexCat(")");
		_FiltLexRegionEnd(0);
		BEGIN INITIAL;
}
<REG>{OP}	{ 
		_FiltLexRegionEnd(0);
		_FiltLexOp(yytext);
		BEGIN INITIAL;
}
<REG>{COM}	|
<REG>{EOL}	{ 
		_FiltLexRegionEnd(1);
		BEGIN INITIAL;
}

{NAME}{SP}/={SP}\(?({NUM}|:{NUM}) {
		if( YY_START == REG ) _FiltLexRegionEnd(1);
		/* Found a range-list */
		BEGIN RLIST;
		narg = 0;
		_FiltLexTypeSet(TOK_RLIST);
		nrange = 0;
		if( !(rangename=FilterSymbolEnter(filt, yytext)) )
		  _filterror("missing column or header parameter");
}
<RLIST>\(	    { ; }
<RLIST>{NUM}:{NUM}  { _FiltLexRangeAdd(rangename, yytext, 3, nrange++); }
<RLIST>:{NUM}       { _FiltLexRangeAdd(rangename, yytext, 1, nrange++); }
<RLIST>{NUM}:	    { _FiltLexRangeAdd(rangename, yytext, 4, nrange++); }
<RLIST>{NUM} 	    { _FiltLexRangeAdd(rangename, yytext, 2, nrange++); }
<RLIST>=	    { ; }
<RLIST>,	    { ; }
<RLIST>{SEP}{SP}/({NAME}|{NUM}{OP}|@{FILE}) {
		/* finish last state */
		_FiltLexNew(0);
		BEGIN INITIAL;
}

{NAME}{SP}/\(	{ 
		/* Found a routine */
		laststart = YY_START;
		BEGIN RTINE;
		narg = 0;
		nrparen = 0;
		_FiltLexTypeSet(TOK_RTINE);
		_FiltLexCat(FilterLexRoutine1(filt, yytext));
		nroutine++;
}
<RTINE>,	{
		/* explicit rule avoids the  _FiltLexNew of general rule */
		_FiltLexCat(",");
}
<RTINE>\(	{
		nrparen++;
		_FiltLexCat("(");
}
<RTINE>\)	{
		nrparen--;
		if( nrparen ){
  		  _FiltLexCat(")");
		}
		else{
		  /* end of routine */
		  BEGIN laststart; 
		  _FiltLexCat(FilterLexRoutine2(filt, ")"));
		}
}

@/{FILE}	{
		if( YY_START == REG ) _FiltLexRegionEnd(1);
		/* Found an include file */
		BEGIN INCLUDE;
}
<INCLUDE>{FILE} {
		char *s, *t;
		if ( include_stack_ptr >= MAX_INCLUDE_DEPTH ) {
		  _filterror("include files are nested too deeply");
		}
		if( (s = (char *)FileContents(yytext, 0, NULL)) == NULL ){
		  _filterror("can't access include file");
		}
		else {
		  t = FilterClip(s);
		  free(s);
		  include_stack[include_stack_ptr++] = YY_CURRENT_BUFFER;
		  BEGIN INITIAL;
		  yy_scan_string(t);
		  BEGIN INITIAL;
		  free(t);
		}
}

{NAME}		{
		if( !(tstr=FilterSymbolEnter(filt, yytext)) )
		  _filterror("missing column or header parameter");
		else{
		  narg++;
		  /* add if initial condition (not region, routine, etc.) */
		  if( YY_START == INITIAL ) _FiltLexTypeSet(TOK_NAME);
		  _FiltLexCat(FilterLexName(filt,tstr));
		}
}

{SP} 		{ ; /* ignore white space */ }

{STR}		{ narg++; _FiltLexCat(yytext); }

{NUM}		{ narg++; _FiltLexCat(yytext); }

{OP}    	{
                if( nfiltseg >0 && filtseg[nfiltseg-1]->multi )
		  _filterror("'panda' or accelerator must be at the end of a boolean expression; found\n");
		_FiltLexOp(yytext);
		if( YY_START != RTINE ) BEGIN INITIAL;
}

\(		{
		nparen++;
		_FiltLexCat(yytext); 
}
\)		{
		nparen--;
		_FiltLexCat(yytext); 
		if( extranew ){
		  _FiltLexNew(1);
		  extranew = 0;
		}
}
{SEP}		{       
		_FiltLexNew(0);
		if( nparen ){
		  extranew = 1;
		}
		BEGIN INITIAL;
}

.		{       _FiltLexCat(yytext); }

<<EOF>>		{
		if ( --include_stack_ptr < 0 ){
	          _FiltLexNew(0);
		  _FiltLexMake();
                  yyterminate();
		} else {
		  yy_delete_buffer( YY_CURRENT_BUFFER );
		  yy_switch_to_buffer(include_stack[include_stack_ptr] );
		}
}

%%

/*
 *
 * Private Routines 
 *
 *
 */

/*
 *
 * _FiltLexNew -- finish last start and set up new filter segment
 *
 */
#ifdef YY_USE_PROTOS
static void
_FiltLexNew(int flag)
#else
static void _FiltLexNew(flag)
	int flag;
#endif
{
  int i;
  int last;
  
  if( nfiltseg >= maxfiltseg ){
    last = maxfiltseg;
    if( maxfiltseg == 0 ){
      maxfiltseg = SEGINC;
      filtseg = (FiltSeg *)malloc(maxfiltseg * sizeof(FiltSeg));
    }
    else{
      maxfiltseg += SEGINC;
      filtseg = (FiltSeg *)realloc(filtseg, maxfiltseg * sizeof(FiltSeg));
    }
    /* clear the unused records */ 
    for(i=last; i<maxfiltseg; i++){
      filtseg[i] = NULL;
    }
  }
  if( !nfiltseg || (filtseg[nfiltseg-1] && filtseg[nfiltseg-1]->s) ){
    filtseg[nfiltseg] = (FiltSeg)calloc(1, sizeof(FiltSegRec));
    nfiltseg++;
  }
  filtseg[nfiltseg-1]->flag = flag;
  filtseg[nfiltseg-1]->n = nfiltseg-1;
}

/*
 *
 * __FiltLexCat -- add a string to the filter string buffer
 *
 */
#ifdef YY_USE_PROTOS
static void
__FiltLexCat(char *str, char **ostr, int *olen)
#else
static void __FiltLexCat(str, ostr, olen)
     char *str;
     char **ostr;
     int *olen;
#endif
{
  int blen;
  int slen;
  
  if( (str == NULL) || (*str == '\0') )
    return;
  else
    slen = strlen(str) + 1;
  
  if( (*ostr == NULL) || (**ostr == '\0') )
    blen = 0;
  else
    blen = strlen(*ostr);
  
  while( (blen + slen) >= *olen ){
    *olen += SEGINC;
  }
  if( blen == 0 )
    *ostr = (char *)calloc(*olen, sizeof(char));
  else
    *ostr = (char *)realloc(*ostr, *olen);
  strcat(*ostr, str);
}

/*
 *
 * _FiltLexCat -- add a string to the filter string buffer
 *
 */
#ifdef YY_USE_PROTOS
static void
_FiltLexCat(char *str)
#else
static void _FiltLexCat(str)
     char *str;
#endif
{
  char *s;
  if( !str || !*str )
    return;
  if( nfiltseg ==0 )
    _FiltLexNew(0);
  s = malloc(strlen(str)+1);
  nowhite(str, s);
  __FiltLexCat(s, &(filtseg[nfiltseg-1]->s), &(filtseg[nfiltseg-1]->len));
  if( s ) free(s);
}

/*
 *
 * _FiltLexBase -- get info from current filtseg
 *
 */
#ifdef YY_USE_PROTOS
static char *
_FiltLexBase(double *d, int nd, int *type, int *op, int ff)
#else
static char *_FiltLexBase(d, nd, type, op, ff)
	double *d;
	int nd;
	int *type;
	int *op;
	int ff;
#endif
{
  char *base;
  char *s, *t;
  int i;
  int got=0;

  /* find base and the 2 limit values */
  base = NewString(filtseg[nfiltseg-1]->s);
  for(i=nd-1; i>=0; i--){
    /* look for last comma */
    t = strrchr(base, ',');
    if( !t ) goto done;
    /* null it out from base string */
    *t = '\0';
    /* point past comma and get double value */
    t++;
    d[i] = strtod(t, &s);
    if( t == s ) goto done;
  }
  /* fill in optional values */
  if( type ){
    *type = filtseg[nfiltseg-1]->type;
  }
  if( op ){
    *op = filtseg[nfiltseg-1]->ops;
  }
  /* destroy the last segment, we will re-create it in a moment */
  if( ff && filtseg[nfiltseg-1] ){
    if( filtseg[nfiltseg-1]->s ) free(filtseg[nfiltseg-1]->s);
    free(filtseg[nfiltseg-1]);
    filtseg[nfiltseg-1] = NULL;
    nfiltseg--;
  }
  got = 1;

done:
  if( !got ){
    free(base);
    base = NULL;
  }
  return base;
}

/*
 *
 * _FiltLexAccel -- appply acceleration to a segment
 *
 */
#ifdef YY_USE_PROTOS
static int
_FiltLexAccel(int n)
#else
static int _FiltLexAccel(n)
     int n;
#endif
{
  char *base=NULL;
  char tbuf[SZ_LINE];
  double d[2];
  double dinc;
  int i;
  int type;
  int ops;
  int got = 0;

  /* finish off last region segment */
  _FiltLexRegionEnd(0);
  /* get base, delete old segment, start a new segment */
  if( !(base=_FiltLexBase(d, 2, &type, &ops, 1)) ) goto done;
  dinc = (d[1] - d[0])/(double)n;
  for(i=1; i<=n; i++){
    /* make a new segment */
    _FiltLexNew(0);
    /* restore type, ops from prev */
    filtseg[nfiltseg-1]->type = type;
    filtseg[nfiltseg-1]->ops = ops;
    /* we are generating multiple regions */
    filtseg[nfiltseg-1]->multi = 1;
    /* add beginning of new region, enclosed in parens */
    if( *base ){
      _FiltLexCat("(");
      _FiltLexCat(base);
    }
    /* add new x and y */
    sprintf(tbuf, DFMT2, d[0]+((i-1)*dinc), d[0]+(i*dinc));
    _FiltLexCat(tbuf);
    _FiltLexCat(")");
    if( *base ){
      _FiltLexCat(")");
    }
  }
  got = 1;

done:
  if( base ) free(base);
  return got;
}


/*
 *
 * _FiltLexVarArgs -- handle varargs in annulus and polygon specially
 *
 */
#ifdef YY_USE_PROTOS
static void
_FiltLexVarArgs(void)
#else
static void _FiltLexVarArgs()
#endif
{
  int n;
  int type, ops;
  double d[2];
  char *base;
  char tbuf[SZ_LINE];

  /* determine how many args are in the varargs specification */
  switch(regid){
  case TY_POINT:
    n = 2;
    break;
  case TY_ANNULUS:
  case TY_PIE:
    n = 4;
    break;
  default:
    narg++;
    return;
  }

  /* if n == narg, we are about to process a varargs value */
  if( narg == n ){
    base = _FiltLexBase(d, 2, &type, &ops, 0);
    switch(n){
    /* points */
    case 2:
      /* finish off last region segment */
      _FiltLexRegionEnd(0);
      /* make a new segment */
      _FiltLexNew(0);
      /* restore type, ops from prev */
      filtseg[nfiltseg-1]->type = type;
      filtseg[nfiltseg-1]->ops = ops;
      /* we are generating multiple regions */
      filtseg[nfiltseg-1]->multi = 1;
      /* add beginning of new region */
      _FiltLexCat(base);
      /* reset arg count */
      narg = 0;
      break;
    /* pie, annuli */
    case 4:
      /* finish off last region segment */
      _FiltLexRegionEnd(0);
      /* make a new segment */
      _FiltLexNew(0);
      /* restore type, ops from prev */
      filtseg[nfiltseg-1]->type = type;
      filtseg[nfiltseg-1]->ops = ops;
      /* we are generating multiple regions */
      filtseg[nfiltseg-1]->multi = 1;
      /* add beginning of new region */
      _FiltLexCat(base);
      /* add previous y as new x */
      sprintf(tbuf, DFMT1, d[1]);
      _FiltLexCat(tbuf);
      /* reset arg count to have the x arg */
      narg = 3;
      break;
    }
    if( base ) free(base);		  
    BEGIN REG;
  }
  /* now we can inc number of args for this region */
  narg++;
}

/*
 *
 * _FiltLexTypeSet -- add type info to the filter segment record
 *
 */
#ifdef YY_USE_PROTOS
static void
_FiltLexTypeSet(int type)
#else
static void _FiltLexTypeSet(type)
     int type;
#endif
{
  if( nfiltseg ==0 )
    _FiltLexNew(0);
  filtseg[nfiltseg-1]->type |= type;
}

/*
 *
 * _FiltLexOp -- add an operator to the filter segment record
 *
 */
#ifdef YY_USE_PROTOS
static void
_FiltLexOp(char *op)
#else
static void _FiltLexOp(op)
     char *op;
#endif
{
  _FiltLexCat(op);
  filtseg[nfiltseg-1]->ops += 1;
}

#ifdef YY_USE_PROTOS
static void
_FiltLexRegionStart(int docols)
#else
static void _FiltLexRegionStart(docols)
	int docols;
#endif
{
  char *s, *t;

  narg = 0;
  s = malloc(strlen(yytext)+1);
  nowhite(yytext, s);
  if( *s == '+' ){
    regtype = TOK_IREG;
    strcpy(regname, s+1);
  }
  else if( (*s == '-') || (*s == '!') ){
    regtype = TOK_EREG;
    strcpy(regname, s+1);
  }
  else{
    regtype = TOK_IREG;
    strcpy(regname, s);
  }
  _FiltLexTypeSet(regtype);
  if( !(t=strchr(regname, '(')) ){
    nsparen=0;
  }
  /* remove the paren from the regname string -- we will explicitly add it */
  else{
    *t = '\0';
    nsparen=1;
  }  
  /* convert to lower case, remove white space */
  culc(regname);
  nowhite(regname, s);
  strcpy(regname, s);
  /* convert abbrev to region */
  if( !strncmp(regname, "ann", 3) ){
    strcpy(regname, "annulus");
    regid = TY_ANNULUS;
  }
  else if( !strncmp(regname, "box", 3) ){
    strcpy(regname, "box");
    regid = TY_BOX;
  }
  else if( !strncmp(regname, "cir", 3) ){
    strcpy(regname, "circle");
    regid = TY_CIRCLE;
  }
  else if( !strncmp(regname, "ell", 3) ){
    strcpy(regname, "ellipse");
    regid = TY_ELLIPSE;
  }
  else if( !strncmp(regname, "fie", 3) ){
    strcpy(regname, "field");
    regid = TY_FIELD;
  }
  else if( !strncmp(regname, "lin", 3) ){
    strcpy(regname, "line");
    regid = TY_LINE;
  }
  else if( !strncmp(regname, "pan", 3) ){
    strcpy(regname, "panda");
    regid = TY_PANDA;
  }
  else if( !strncmp(regname, "pie", 3) ){
    strcpy(regname, "pie");
    regid = TY_PIE;
  }
  else if( !strncmp(regname, "poi", 3) ){
    strcpy(regname, "point");
    regid = TY_POINT;
  }
  else if( !strncmp(regname, "pol", 3) ){
    strcpy(regname, "polygon");
    regid = TY_POLYGON;
  }
  switch( filt->type ){
  case TYPE_EVENTS:
    /* normally, we filter events analytically using evregions.o */
    if( !filt->evsect )
      _FiltLexCat("ev");
    /* if evsect=xxx is specified, we filter by image pixels */
    else
      _FiltLexCat("im");
    break;
  case TYPE_IMAGE:
    /* image are filtered by image pixels */
    _FiltLexCat("im");
    break;
  }
  _FiltLexCat(FilterLexRegion1(filt,regname));
  _FiltLexCat("(g,$NREGION,$NSHAPE,");
  /* add include/exclude flag */
  if( regtype ==  TOK_IREG )
    _FiltLexCat("1");
  else
    _FiltLexCat("0");
  switch(docols){
  case 0:
    /* initialize default columns for tables -- but don't enter symbols yet */
    FilterSymbolDefaults(filt, 0);
    /* initialize wcs for default image case */
    _FiltLexWcsCols(filt->fhd, filt->xbin, filt->ybin);
    _FiltLexCat(",(double)x,(double)y");
    break;
  case 1:
    if( !initbindefs ){
      /* initialize default columns for tables and enter symbols */
      FilterSymbolDefaults(filt, 1);
      /* initialize wcs for default table case */
      _FiltLexWcsCols(filt->fhd, filt->xbin, filt->ybin);
      initbindefs++;
    }
    if( !filt->xbin || !filt->ybin )
      _filterror("no default columns defined for region");
    _FiltLexCat(",(double)");
    _FiltLexCat(filt->xbin);
    _FiltLexCat(",(double)");
    _FiltLexCat(filt->ybin);
    break;
  case 2:
    _FiltLexCat(",(double)");
    _FiltLexCat(colnames[0]);
    _FiltLexCat(",(double)");
    _FiltLexCat(colnames[1]);
    break;
  }
  if( s ) free(s);
}

#ifdef YY_USE_PROTOS
static void
_FiltLexRegionEnd(int flag)
#else
static void
_FiltLexRegionEnd(flag)
	int flag;
#endif
{
  /* for polygons, we need to signal end of varargs */
  if( regid == TY_POLYGON ){
    sprintf(sbuf, DFMT2, PSTOP, PSTOP);
    _FiltLexCat(sbuf);
  }
  else if( (regid == TY_BOX) && (narg==4) ){
    _FiltLexCat(",0.0"); narg++;
  }
  else if( (regid == TY_ELLIPSE) && (narg==4) ){
    _FiltLexCat(",0.0"); narg++;
  }
  _FiltLexArgCheck(regname, narg);
  _FiltLexCat(FilterLexRegion2(filt,")"));
  /* for panda, we need convert to a set of pie/annulus intersections */
  if( !strcmp(regname, "panda") ){
    _FiltLexRegionPanda();
  }
  if( flag )
    _FiltLexNew(0);
  /* no column arguments */
  colargs = 0;
}

#ifdef YY_USE_PROTOS
static int
_FiltLexRegionPanda(void)
#else
static int
_FiltLexRegionPanda()
#endif
{
  char *b=NULL;
  char *base=NULL;
  char arglist[SZ_LINE];
  char pref[SZ_LINE];
  char pref2[SZ_LINE];
  char tbuf[SZ_LINE];
  double d[8];
  double pinc, ainc;
  int p, a;
  int phi, ahi;
  int type;
  int ops;
  int got = 0;

  /* get base, delete old segment, start a new segment */
  if( !(base=_FiltLexBase(d, 8, &type, &ops, 1)) ) goto done;
  /* prepend whatever came before the region */
  if( (b = strstr(base, regname)) ){
    /* back up before the 2-char prefix (ev or im) */
    b -= 2;
    strncpy(pref2, b, 2);
    pref2[2] = '\0';
    strncpy(pref, base, b-base);
    pref[b-base] = '\0';
    /* get common arglist */
    if( (b = strchr(b, '(')) )
      strcpy(arglist, ++b);
    else
      goto done;
  }
  else
    goto done;
  /* make sure we have at least one pie and annulus */
  if( d[4] <= 0 ){
    d[2] = 0;
    d[3] = 360;
    d[4] = 1;
  }
  if( d[7] <= 0 ){
    d[5] = 0;
    d[6] = LARGE_NUMBER;
    d[7] = 1;
  }
  pinc = (d[3] - d[2])/d[4];
  phi = (int)d[4];
  ainc = (d[6] - d[5])/d[7];
  ahi = (int)d[7];
  for(p=1; p<=phi; p++){
    for(a=1; a<=ahi; a++){
      /* make a new segment */
      _FiltLexNew(0);
      /* restore type, ops from prev */
      filtseg[nfiltseg-1]->type = type;
      filtseg[nfiltseg-1]->ops = ops;
      /* we are generating multiple regions */
      filtseg[nfiltseg-1]->multi = 1;
      /* generate segment string */
      sprintf(tbuf,
          "%s%spie(%s,%f,%f,%f,%f)&&%sannulus(%s,%f,%f,%f,%f)",
	  pref,
	  pref2, arglist, d[0], d[1], d[2]+((p-1)*pinc), d[2]+(p*pinc),
          pref2, arglist, d[0], d[1], d[5]+((a-1)*ainc), d[5]+(a*ainc));
      _FiltLexCat(tbuf);
    }
  }
  got = 1;

done:
  if( base ) free(base);
  return got;
}

/*
 *
 * _FiltLexRangeAdd -- add a rangelist string to the filter string buffer
 *
 */
#ifdef YY_USE_PROTOS
static void
_FiltLexRangeAdd(char *xname, char *str, int type, int n)
#else
static void _FiltLexRangeAdd(xname, str, type, n)
     char *xname;
     char *str;
     int type;
     int n;
#endif
{
  char tbuf[SZ_LINE];
  char *t=tbuf;
  char *s=str;
  char *name;
  
  /* get "in-expression" name for current technique */
  name = FilterLexName(filt, xname);
  
  /* if this is not the beginning, we first need to OR with prev */
  if( n != 0 ){
    strcpy(t, "||");
    while( *t ) t++;
  }
  
  /* put in parens for safety */
  strcpy(t, "(");
  while( *t ) t++;
  
  /* add the name */
  strcpy(t, name);
  while( *t ) t++;
  
  /* add the rangelist */
  switch(type){
  case 1:
    strcpy(t, "<=");
    while( *t ) t++;
    /* skip past : */
    s++;
    /* append upper value */
    strcpy(t, s);
    while( *t ) t++;
    break;
  case 2:
    strcpy(t, "==");
    while( *t ) t++;
    /* append value */
    strcpy(t, s);
    while( *t ) t++;
    break;
  case 3:
    strcpy(t, ">=");
    while( *t ) t++;
    /* append lower value */
    while( *s && (*s != ':') ) *t++ = *s++;
    /* skip past : */
    s++;
    strcpy(t, "&&");
    while( *t ) t++;
    strcpy(t, name);
    while( *t ) t++;
    strcpy(t, "<=");
    while( *t ) t++;
    /* append upper value */
    while( *s ) *t++ = *s++;
    break;
  case 4:
    strcpy(t, ">=");
    while( *t ) t++;
    /* append lower value */
    while( *s && (*s != ':') ) *t++ = *s++;
    break;
  }
  strcpy(t, ")");
  _FiltLexCat(tbuf);
}

/*
 *
 * _FiltLexTypeCmp -- compare routine for qsort
 *
 */
#ifdef YY_USE_PROTOS
static int
_FiltLexTypeCmp(const void *s1, const void *s2)
#else
static int _FiltLexTypeCmp(s1, s2)
     const void *s1;
     const void *s2;
#endif
{
  FiltSeg *f1 = (FiltSeg *)s1;
  FiltSeg *f2 = (FiltSeg *)s2;
  
  /* if we have flag set, we have an operator, and it must be last */
  if( (*f1)->flag != (*f2)->flag ){
    if( (*f1)->flag )                   return 1;
    else                                return -1;
  }
  /* we want descending order so that eregions are last */
  if( (*f1)->type < (*f2)->type )       return 1;
  else if( (*f1)->type > (*f2)->type )  return -1;
  /* as a last resort, try to keep them in the order in which we started */
  else if( (*f1)->n < (*f2)->n )        return -1;
  else if( (*f1)->n > (*f2)->n )        return 1;
  else                                  return 0;
}

/*
 *
 * _FiltLexTypeMatch -- see if the type of filtering we are doing
 * matches the type of token we have
 *
 */
#ifdef YY_USE_PROTOS
int
_FiltLexTypeMatch(int ftype, int ttype)
#else
int _FiltLexTypeMatch(ftype, ttype)
     int ftype;
     int ttype;
#endif
{
  switch(ftype){
  case TYPE_IMAGE:
    /* we don't allow range list and col names in image filters */
    if( (ttype & TOK_RLIST) || (ttype & TOK_NAME) )
      return 0;
    /* everything else if fine, if it hasa region in it */
    else if( (ttype & TOK_IREG) || (ttype & TOK_EREG) )
      return 1;
    else
      return 0;
    break;
  case TYPE_EVENTS:
    return 1;
    break;
  default:
    return 1;
    break;
  }
}

#ifdef YY_USE_PROTOS
static void
_FiltLexArgCheck(char *name, int args)
#else
static void _FiltLexArgCheck(name, args)
	char *name;
        int args;
#endif
{
  char ebuf[SZ_LINE];
  int i;

  for(i=0; rnames[i]; i++){
    if( !strcmp(name, rnames[i]) ){
      if( (rargs[i] >= 0) && (args != rargs[i]) ){
	*yytext = '\0';
        sprintf(ebuf, "'%s' requires %d arguments (%d found)",
                rnames[i], rargs[i], args);
	_filterror(ebuf);
      }
      break;
    }
  }
}

#ifdef YY_USE_PROTOS
static void
_FiltLexSetWcs(char *s)
#else
static void _FiltLexSetWcs(s)
	char *s;
#endif
{
  char name[SZ_LINE];
  int ip=0;

  newdtable(",=;");
  /* get name of global coordsys */
  if( word(s, name, &ip) ){
    if( gcoordsys ) free(gcoordsys);
    gcoordsys = NewString(name);
    culc(gcoordsys);
  }
  /* restore old delim table */
  freedtable();
}

#ifdef YY_USE_PROTOS
static char *
_FiltLexCB(char *name, void *client_data)
#else
static char *_FiltLexCB(name, client_data)
	char *name;
	void *client_data;
#endif
{
  if( !strcmp(name, "NREGION") ){
    sprintf(macrobuf, "%d", nregion);
    return macrobuf;
  }
  else if( !strcmp(name, "NSHAPE") ){
    nshape++;
    sprintf(macrobuf, "%d", nshape);
    return macrobuf;
  }
  else{
    return NULL;
  }
}

/*
 *
 * _FiltLexParen -- see if parens are balanced
 *
 */
#ifdef YY_USE_PROTOS
static int
_FiltLexParen(char *s)
#else
static int _FiltLexParen(s)
	char *s;
#endif
{
  char *t;
  int got;
  if( !s || !*s ) return 0;
  for(got=0, t=s; *t; t++){
    if( *t == '(' ) got++;
    if( *t == ')' ) got--;
  }
  return got;
}

/*
 *
 * _FiltLexAddOp -- determine if we add a connecting operator for this segment
 *
 */
#ifdef YY_USE_PROTOS
static int
_FiltLexHasOp(int i, char *s)
#else
static int _FiltLexHasOp(i, s)
	int i;
	char *s;
#endif
{
  while( (*s == '(') || (*s == ')') ) s++;
  switch(*s){
  case '&':
  case '|':
  case '=':
  case '!':
  case '<':
  case '>':
    return 1;
  default:
    return 0;
  }
}

/*
 *
 * _FiltLexMake -- generate filter from filter segments
 *
 */
#ifdef YY_USE_PROTOS
static void
_FiltLexMake(void)
#else
static void _FiltLexMake()
#endif
{
  int i;
  int type;
  int addparen;
  char *s;

  /* we want descending order so that eregions are last in the expression */
  qsort(filtseg, nfiltseg, sizeof(FiltSeg), _FiltLexTypeCmp);
  /* run through segments, inserting proper boolean connectors */
  for(i=0, segid=0, nregion=0; i<nfiltseg; ){
    if( filtseg[i] && filtseg[i]->s ){
      type = filtseg[i]->type;
      /* if its just the end of previous statment (perhaps just ')',
	 add it to the output string and jump out */
      if( !type ){
        __FiltLexCat(filtseg[i]->s, &filter, &filtlen);
	i++;
	continue;
      }
      /* this is where we make a final count of regions */
      if( type & (TOK_IREG|TOK_EREG) ){
        nregion++;
      }
      /* make sure we want the type being offered */
      if( !_FiltLexTypeMatch(filt->type, type) ){
	i++;
        continue;
      }
      /* another segment -- we inc before expanding the macros, since
         its the macro callback that uses this value */
      s = ExpandMacro(filtseg[i]->s, NULL, NULL, 0, _FiltLexCB, filtseg[i]);
      if( segid++ && !_FiltLexHasOp(i, s) ){
	if( type & TOK_IREG )
	  __FiltLexCat("||", &filter, &filtlen);
	else
	  __FiltLexCat("&&", &filter, &filtlen);
      }
      if( !_FiltLexParen(filter) && !_FiltLexParen(s) && !filtseg[i]->flag )
	addparen = 1;
      else
	addparen = 0;
      if( addparen )
        __FiltLexCat("(", &filter, &filtlen);
      __FiltLexCat(s, &filter, &filtlen);
      if( addparen )
        __FiltLexCat(")", &filter, &filtlen);
      if( s ) free(s);
      i++;
      for(; (i<nfiltseg) && (filtseg[i]->type==type); ){
        /* another segment -- we inc before expanding the macros, since
           its the macro callback that uses this value */
        segid++;
	nregion++;
        s = ExpandMacro(filtseg[i]->s, NULL, NULL, 0, _FiltLexCB, filtseg[i]);
	if( !filtseg[i]->flag && !_FiltLexHasOp(i, s) ){
	  if( type & TOK_IREG )
	    __FiltLexCat("||", &filter, &filtlen);
	  else
	    __FiltLexCat("&&", &filter, &filtlen);
        }
        if( !_FiltLexParen(filter) && !_FiltLexParen(s) && !filtseg[i]->flag )
	  addparen = 1;
        else
	  addparen = 0;
        if( addparen )
          __FiltLexCat("(", &filter, &filtlen);
        __FiltLexCat(s, &filter, &filtlen);
        if( addparen )
          __FiltLexCat(")", &filter, &filtlen);
        if( s ) free(s);
	i++;
      }
    }
    /* ignore invalid arguments */
    else{
      i++;
    }
  }
}

/*
 *
 * _FiltLexEnd -- cleanup from parsing regions
 *
 */
#ifdef YY_USE_PROTOS
static void
_FiltLexEnd(void)
#else
static void _FiltLexEnd()
#endif
{
  int i;

  /* reset state variables */
  colargs = 0;
  initbindefs = 0;
  include_stack_ptr = 0;
  nroutine = 0;
  nshape = 0;
  nparen = 0;
  extranew = 0;
  parse_error = 0;
  filtlen = 0;
  regid = 0;
  regtype = 0;
  *regname = '\0';
  *(wcsnames[0]) = '\0';
  *(wcsnames[1]) = '\0';
  /* free up previous filter */
  if( filter ){
    free(filter);
    filter = NULL;
  }
  /* free up wcs */
  if( wcs ){
    wcsfree (wcs);
    wcs = NULL;   
  }
  /* no segments */
  if( filtseg ){
    for(i=0; i<nfiltseg; i++){
      if( filtseg[i] ){
        if( filtseg[i]->s ) free(filtseg[i]->s);
        free(filtseg[i]);
      }
    }
    free(filtseg);
    filtseg = NULL;
  }
  maxfiltseg = 0;
  nfiltseg = 0;
  if( gcoordsys ){
    free(gcoordsys);
    gcoordsys = NULL;
  }
  if( fcoordsys ){
    free(fcoordsys);
    fcoordsys = NULL;
  }
}

/*
 *
 * _FiltLexGetWcs -- determine the WCS coord system to use for wcsc translation
 *
 */
#ifdef YY_USE_PROTOS
static char *
_FiltLexGetWcs(void)
#else
static char *_FiltLexGetWcs()
#endif
{
  if( gcoordsys &&
	strcmp(gcoordsys, "image") && strcmp(gcoordsys, "physical") )
    return gcoordsys;
  if( fcoordsys &&
	strcmp(fcoordsys, "image") && strcmp(fcoordsys, "physical") )
    return fcoordsys;
  return NULL;
}

/*
 *
 * _FiltLexWcsCols -- reset WCS for these columns
 *
 */
#ifdef YY_USE_PROTOS
static void
_FiltLexWcsCols(FITSHead iheader, char *name0, char *name1)
#else
static void _FiltLexWcsCols(iheader, name0, name1)
	FITSHead iheader;
	char *name0;
	char *name1;
#endif
{
  int i, j;
  int simple=1;
  double dval, dval2;
  char *s;
  FITSHead oheader;    
  FITSCard card;
  
  /* make sure we have something to process */
  if( !iheader )
    return;

  if( filt->fhd->image ){
    /* free up old wcs */
    if( wcs ) wcsfree (wcs);
    /* init new wcs */
    hlength(ft_cards(filt->fhd),0);
    wcs = wcsinit(ft_cards(filt->fhd));
    tltyp[0] = 0;
    tltyp[1] = 0;
  }
  /* for tables, we can init wcs only the specified columns */
  else if( name0 && *name0 && name1 && *name1 ){
    /* see if these are the same as the last set of columns */
    if( (*(wcsnames[0]) && !strcasecmp(name0,wcsnames[0])) && 
        (*(wcsnames[1]) && !strcasecmp(name1,wcsnames[1])) && wcs ){
      return;
    }
    /* free up old wcs */
    if( wcs ) wcsfree (wcs);
    /* make a copy of the header */
    oheader = ft_headcopy(iheader);
    /* make sure this is a primary FITS file */
    ft_cardfmt((FITSCard)oheader->cards, 
	       "SIMPLE", 0, FT_LOGICAL, &simple, 0, "Standard FITS");
    /* transform WCS cards for the specified columns, if we have such */
    for(j=1, i=1; i<=iheader->table->tfields; i++){
      if( !strcasecmp(iheader->table->col[i-1].name, name0) ||
          !strcasecmp(iheader->table->col[i-1].name, name1) ){
        dval = ft_headgetr(iheader, "TLMIN", i, 1.0, &card);
        /* set tlmin for the appropriate column */
	if( !strcasecmp(iheader->table->col[i-1].name, name0) ){
          tlmin[0] = dval;
          tltyp[0] = iheader->table->col[i-1].type;
        }
        else{
          tlmin[1] = dval;
          tltyp[1] = iheader->table->col[i-1].type;
        }
        if( (s=ft_headgets(iheader, "TCTYP", i, NULL, &card)) && card ){
          ft_headapps(oheader, "CTYPE", j, s, NULL);
          free(s);
        }
        if( (s=ft_headgets(iheader, "TCRVL", i, NULL, &card)) && card ){
          ft_headappv(oheader, "CRVAL", j, s, NULL);
          free(s);
        }
        if( (s=ft_headgets(iheader, "TCDLT", i, NULL, &card)) && card ){
          ft_headappv(oheader, "CDELT", j, s, NULL);
          free(s);
        }
        if( (s=ft_headgets(iheader, "TCRPX", i, NULL, &card)) && card ){
          /* modify WCS center coords based on value of TLMIN */
          dval2 = strtod(s,NULL) + 1 - dval;
          ft_headsetr(oheader, "CRPIX", j, dval2, 7, NULL, 1);
          free(s);
        }
        if( (s=ft_headgets(iheader, "TCROT", i, NULL, &card)) && card ){
          ft_headappv(oheader, "CROTA", j, s, NULL);
          free(s);
        }
        j++;
      }
    }
    /* init new wcs */
    hlength(ft_cards(oheader),0);
    wcs = wcsinit(ft_cards(oheader));
    ft_headfree(oheader, 1);
    /* save names for next time */
    strcpy(wcsnames[0], name0);
    strcpy(wcsnames[1], name1);
  }

  /* set some extra wcs parameters */
  if( wcs && iswcs(wcs) ){
    if( fcoordsys ) free(fcoordsys);
    fcoordsys = NewString(wcs->radecin);
    culc(fcoordsys);
    if( !wcs->coorflip ){
      cdelt1 = wcs->cdelt[0];
      cdelt2 = wcs->cdelt[1];
    }
    else{
      cdelt1 = wcs->cdelt[1];
      cdelt2 = wcs->cdelt[0];
    }
  }
  /* set global coord system first time through */
  if( !gcoordsys ){
    if( (gcoordsys=NewString(getenv("COORDSYS"))) == NULL )
      gcoordsys = NewString(DEFAULT_COORDSYS);
  }
}

/*
 *
 * _FiltLexLcx -- determine if we have a linear-type WCS
 *
 */
#ifdef YY_USE_PROTOS
static int
_FiltLexLcx(void)
#else
static int _FiltLexLcx()
#endif
{
  char *coordsys;
  coordsys = gcoordsys;
  if( !coordsys || !*coordsys )
    return 0;
  if( !strcasecmp(coordsys, "image")     ) return LCX_IMAGE;
  if( !strcasecmp(coordsys, "physical")  ) return LCX_PHYS;
  if( !strcasecmp(coordsys, "amplifier") ) return LCX_AMP;
  if( !strcasecmp(coordsys, "detector")  ) return LCX_DET;
  return 0;
}

#ifdef YY_USE_PROTOS
static void
_FiltLexNum(int lcx)
#else
static void _FiltLexNum(lcx)
	int lcx;
#endif
{
  /* for linear coordinate systems (physical, amps, etc.),
     we have to convert to image coords or physical coords,
     depending on the type of data and type of filtering */
  if( lcx ){
    /* process table data in image or physical coords */
    if( filt->fhd->table ){
      if( PHYSCOORDS ){
        /* convert image positions to physical coords */
	if( (lcx==LCX_IMAGE) && POSARG ){
	  dval = tli2p(strtod(yytext,NULL),tlmin[narg%2],tltyp[narg%2]);
	  sprintf(sbuf, DFMT1, dval);
	  _FiltLexCat(sbuf);
        }
	/* everything else is just passed along */
	else{
	  dval = strtod(yytext,NULL);
	  sprintf(sbuf, DFMT1, dval);
	  _FiltLexCat(sbuf);
        }
      }
      /* image & image-style filtering required image coords */
      else{
	/* convert non-image positions to image coords */
	if( (lcx!=LCX_IMAGE) && POSARG ){
	  dval = tlp2i(strtod(yytext,NULL),tlmin[narg%2],tltyp[narg%2]);
	  sprintf(sbuf, DFMT1, dval);
	  _FiltLexCat(sbuf);
        }
	/* everything else is just passed along */
	else{
	  dval = strtod(yytext,NULL);
	  sprintf(sbuf, DFMT1, dval);
	  _FiltLexCat(sbuf);
        }
      }
    }
    /* everything else is in image coords */
    else{
      /* convert positional arguments to image */
      if( POSARG ){
	/* save x value for processing with y next time */
	if( XARG ){
	  xpos = SAOstrtod(yytext,NULL);
	}
	else{
	  ypos = SAOstrtod(yytext,NULL);
	  switch(lcx){
	  case LCX_IMAGE:
	    xpix = xpos;
            ypix = ypos;
            break;
	  case LCX_PHYS:
	    ft_phy2img(filt->fhd, xpos, ypos, &xpix, &ypix);
	    break;
	  case LCX_AMP:
	    ft_amp2phy(filt->fhd, xpos, ypos, &dval, &dval2);
	    ft_phy2img(filt->fhd, dval, dval2, &xpix, &ypix);
	    break;
	  case LCX_DET:
	    ft_det2phy(filt->fhd, xpos, ypos, &dval, &dval2);
	    ft_phy2img(filt->fhd, dval, dval2, &xpix, &ypix);
	    break;
	  }
	  sprintf(sbuf, DFMT2, xpix, ypix);
	  _FiltLexCat(sbuf); 
	}
      }
      /* angle arguments are just passed along */
      else if( ANGARG ){
	dval = strtod(yytext,NULL);
	sprintf(sbuf, DFMT1, dval);
	_FiltLexCat(sbuf); 
      }
      /* convert size args, which may or may not be in pairs */
      else{
	/* arg 1 is associated with ra, arg2 with dec */
	if( XARG ){
	  xpos = ABS(strtod(yytext,NULL));
	  ypos = 0.0;
	  switch(lcx){
	  case LCX_IMAGE:
            xpix = xpos;
            break;
	  case LCX_PHYS:
	    ft_phy2img_size(filt->fhd, xpos, ypos, &xpix, &ypix);
	    break;
	  case LCX_AMP:
	    ft_amp2phy_size(filt->fhd, xpos,ypos, &dval,&dval2);
	    ft_phy2img_size(filt->fhd, dval,dval2, &xpix,&ypix);
	    break;
	  case LCX_DET:
	    ft_det2phy_size(filt->fhd, xpos,ypos, &dval,&dval2);
	    ft_phy2img_size(filt->fhd, dval,dval2, &xpix,&ypix);
	    break;
	  }
	  sprintf(sbuf, DFMT1, xpix);
	}
	else{
	  xpos = 0.0;
	  ypos = ABS(strtod(yytext,NULL));
	  switch(lcx){
	  case LCX_IMAGE:
            ypix = ypos;
            break;
	  case LCX_PHYS:
	    ft_phy2img_size(filt->fhd, xpos, ypos, &xpix, &ypix);
	    break;
	  case LCX_AMP:
	    ft_amp2phy_size(filt->fhd, xpos,ypos, &dval,&dval2);
	    ft_phy2img_size(filt->fhd, dval,dval2, &xpix,&ypix);
	    break;
	  case LCX_DET:
	    ft_det2phy_size(filt->fhd, xpos,ypos, &dval,&dval2);
	    ft_phy2img_size(filt->fhd, dval,dval2, &xpix,&ypix);
	    break;
	  }
	  sprintf(sbuf, DFMT1, ypix);
	}
	_FiltLexCat(sbuf); 
      }
    }
  }
  /* not lcx -- angle arguments are just passed along */
  else if( ANGARG ){
    dval = strtod(yytext,NULL);
    sprintf(sbuf, DFMT1, dval);
    _FiltLexCat(sbuf); 
  }
  /* not lcx -- everything else is assumed to be in units of "degrees"
     in the default system. add 'd' and put back for reparsing */
  else{
    int i;
    char *yycopy = strdup(yytext);
    unput('d');
    for(i=yyleng-1; i>=0; --i)
      unput(yycopy[i]);
    free(yycopy);
    narg--;
  }
}

/*
 *
 * Semi-public Routines 
 *
 *
 */

/*
 *
 * _FilterString -- return output filter from lexer
 *
 */
#ifdef YY_USE_PROTOS
char *
_FilterString(void)
#else
char *_FilterString()
#endif
{
  if( filtlen <= 0 )
    _FiltLexMake();
  return(filter);
}

#ifdef YY_USE_PROTOS
int
FilterRoutineCount(void)
#else
int FilterRoutineCount()
#endif
{
  return(nroutine);
}

#ifdef YY_USE_PROTOS
int
FilterShapeCount(void)
#else
int FilterShapeCount()
#endif
{
  return(nshape);
}

#ifdef YY_USE_PROTOS
int
FilterRegionCount(int type)
#else
int FilterRegionCount(type)
	int type;
#endif
{
  int i, j;
  for(i=0, j=0; i<nfiltseg; i++){
    if( filtseg[i]->type & type ){
      j++;
    }
  }
  return(j);
}

#ifdef YY_USE_PROTOS
int
FilterParseError(void)
#else
int FilterParseError()
#endif
{
  return(parse_error);
}

#ifdef YY_USE_PROTOS
int
FilterTlmin(double *tlmins)
#else
int FilterTlmin(tlmins)
	double *tlmins;
#endif
{
  tlmins[0] = tlmin[0];
  tlmins[1] = tlmin[1];
  return 1;
}

#ifdef YY_USE_PROTOS
int
FilterTltyp(int *tltyps)
#else
int FilterTltyp(tltyps)
	int *tltyps;
#endif
{
  tltyps[0] = tltyp[0];
  tltyps[1] = tltyp[1];
  return 1;
}

#ifdef YY_USE_PROTOS
void
FiltInitParser(void)
#else
void FiltInitParser()
#endif
{
  /* make sure we free'd up all space from last time */
  _FiltLexEnd();
  /* set up some convenience variables */
  filt = FilterDefault();
  BEGIN INITIAL;
  laststart=INITIAL;
}

#ifdef YY_USE_PROTOS
void
FiltRestart(char *s)
#else
void FiltRestart(s)
     char *s;
#endif
{
  yyrestart((FILE *)s);
  BEGIN INITIAL;
}

#ifdef YY_USE_PROTOS
void
FiltScanString(char *s)
#else
void FiltScanString(s)
     char *s;
#endif
{
  int i;
  char *t;
  /* make sure there is an explicit delim at end so we are done before
     lex finds EOF -- otherwise panda shapes don't get processed */
  i = strlen(s);
  t = malloc(i+2);
  strcpy(t, s);
  if( (t[i-1] != ';') && (t[i-1] != '\n') )
    strcat(t, ";");
  yy_scan_string(t);
  if( t ) free(t);
}

/* yyerror -- renamed to _filterror because flex -P does not change this */
#ifdef YY_USE_PROTOS
int
_filterror(char *msg)
#else
int _filterror(msg)
     char *msg;
#endif
{
  if( msg )
    fprintf(stderr,  "ERROR: %s", msg);
  if( yytext && *yytext ){
    if( msg && msg[strlen(msg)-1] != '\n' )
      fprintf(stderr, " ");
    fprintf(stderr, "while parsing filter at '%s'", yytext);
  }
  fprintf(stderr, "\n");
  YY_FLUSH_BUFFER;
  parse_error = 1;  
  yyterminate();
}

#ifdef YY_USE_PROTOS
int yywrap(void)
#else
int yywrap()
#endif
{
  return 1;
}

