%option noyywrap
%option caseless
%option never-interactive
%option c++

%{
  #include <stdio.h>
  #include <stdlib.h>
  #include <string.h>

  #include "util.h"
  #include "othmarkerparser.C.h"

  extern othFlexLexer* othlexx;
%}

%x DISCARD

D   [0-9]
E   [Ee][+-]?{D}+

/* rules */

%%

<DISCARD>[\n]	{ // special case-- #\n
		  BEGIN INITIAL;
		  yyless(0);             // put back the terminator
		  strcpy(othlval.str,""); // feed a blank string
		  return STRING;
		}

<DISCARD>[^;\n]* {  // Discard reset of line
		  BEGIN INITIAL;
		  strcpy(othlval.str,yytext);
		  return STRING;
		}

amplifier		{return AMPLIFIER_;}
arcmin			{return ARCMIN_;}
arcsec			{return ARCSEC_;}
arrow			{return ARROW_;}
b1950			{return B1950_;}
box			{return BOX_;}
boxcircle		{return BOXCIRCLE_;}
circle			{return CIRCLE_;}
color			{return COLOR_;}
coordsys		{return COORDSYS_;}
cross			{return CROSS_;}
debug			{return DEBUG_;}
degrees			{return DEGREES_;}
delete			{return DELETE_;}
detector		{return DETECTOR_;}
diamond			{return DIAMOND_;}
edit			{return EDIT_;}
ellipse			{return ELLIPSE_;}
ecliptic		{return ECLIPTIC_;}
equatorial		{return EQUATORIAL_;}
false			{return FALSE_;}
fixed			{return FIXED_;}
fk4			{return FK4_;}
fk5			{return FK5_;}
font			{return FONT_;}
galactic		{return GALACTIC_;}
global			{return GLOBAL_;}
icrs			{return ICRS_;}
include			{return INCLUDE_;}
image			{return IMAGE_;}
j2000			{return J2000_;}
lin[e]			{return LINE_;}
linear			{return LINEAR_;}
local			{return LOCAL_;}
logical			{return LOGICAL_;}
move			{return MOVE_;}
n			{return N_;}
no			{return NO_;}
off			{return OFF_;}
on			{return ON_;}
physical		{return PHYSICAL_;}
pixels			{return PIXELS_;}
point			{return POINT_;}
polygon			{return POLYGON_;}
rotate			{return ROTATE_;}
rotbox			{return ROTBOX_;}
ruler			{return RULER_;}
select			{return SELECT_;}
text			{return TEXT_;}
true			{return TRUE_;}
version			{return VERSION_;}
x			{return X_;}
y			{return Y_;}
yes			{return YES_;}

[+-]?{D}+	{ // Integer
		  othlval.integer = atoi(yytext);
		  return INT;
		}

[+-]?{D}+"."?({E})? |
[+-]?{D}*"."{D}+({E})? { // Real Number
		  othlval.real = atof(yytext);
		  return REAL;
		}

[+-]?{D}+"."?d |
[+-]?{D}*"."{D}+d { // degrees
		  yytext[yyleng-1] = '\0';
		  othlval.real = atof(yytext);
		  return DEGREE;
		}	

[+-]?{D}+"."?r |
[+-]?{D}*"."{D}+r { // radians
		  yytext[yyleng-1] = '\0';
		  othlval.real = atof(yytext);
		  return RADIAN;
		}

{D}+"."?' |
{D}*"."{D}+'	{ // minutes of arc
		  yytext[yyleng-1] = '\0';
		  othlval.real = atof(yytext);
		  return ARCMINUTE;
		}	

{D}+"."?\" |
{D}*"."{D}+\"	{ // seconds of arc
		  yytext[yyleng-1] = '\0';
		  othlval.real = atof(yytext);
		  return ARCSECOND;
		}

[+-]?{D}+h	{ // hours
		  yytext[yyleng-1] = '\0';
		  othlval.integer = atoi(yytext);
		  return HOUR;
		}	

{D}+m		{ // minutes
		  yytext[yyleng-1] = '\0';
		  othlval.integer = atoi(yytext);
		  return MINUTE;
		}	

{D}+"."?s |
{D}*"."{D}+s	{ // seconds
		  yytext[yyleng-1] = '\0';
		  othlval.real = atof(yytext);
		  return SECOND;
		}

[+-]?{D}+:{D}+:{D}+"."? |
[+-]?{D}+:{D}+:{D}*"."{D}+ { // Sexagesimal
		  strcpy(othlval.str,yytext);
		  return SEXSTR;
		}	

\"[^\"\n]*\" | 
\'[^\'\n]*\'	{ // Quoted String
		  strncpy(othlval.str,yytext+1,yyleng-2); // skip the " " 
	          othlval.str[yyleng-2] = '\0'; // Remove the '"' 
		  return STRING;
		}

\{[^\}\n]*\} 	{ // Quoted String
		  strncpy(othlval.str,yytext+1,yyleng-2); // skip the '{'
	          othlval.str[yyleng-2] = '\0'; // Remove the '}'
		  return STRING;
		}

[0-9A-Za-z]+	{ // General String
		  strcpy(othlval.str,yytext);
		  return STRING;
		}

[ \t]+		{ // White Spaces
		} 

[\n]		{ // line feed
		  return '\n';
		}

.		{ // Else, return the char
		  return yytext[0];
		}

%%

void othDiscard(int doit)
{
  if (othlexx)
    othlexx->begin(DISCARD, doit);
}

void othFlexLexer::begin(int which, int doit)
{
  BEGIN which;
  if (doit)
    yyless(0);
}
