#include "list.h"
#include "vector.h"
#include "marker.h"
#include "callback.h"

template class List<Vertex>;
template class List<Marker>;
template class List<CallBack>;

template<class T> List<T>::List()
{
  head_ = NULL;
  tail_ = NULL;
  current_ = NULL;
  count_ = 0;
}

template<class T> List<T>::~List()
{
  T* ptr = head_;
  while (ptr) {
    T* tmp = ptr->next();
    delete ptr;
    ptr = tmp;
  }
}

template<class T> void List<T>::insertHead(T* p)
{
  if (head_) {
    p->setNext(head_);
    p->setPrevious(NULL);
    head_->setPrevious(p);
    head_ = p;
  }
  else {
    head_ = p;
    tail_ = p;
  }

  current_ = head_;
  count_++;
}

template<class T> void List<T>::insert(int which, T* t)
{
  head();
  for (int i=0; i<which; i++)
    next();

  if (current_) {
    T* n = current_->next();

    t->setPrevious(current_);
    t->setNext(n);

    current_->setNext(t);

    if (n)
      n->setPrevious(t);
    else
      tail_ = t;

    count_++;
  }
}

template<class T> void List<T>::append(T* p)
{
  if (tail_) {
    p->setPrevious(tail_);
    p->setNext(NULL);
    tail_->setNext(p);
    tail_ = p;
  }
  else {
    head_ = p;
    tail_ = p;
  }

  current_ = tail_;
  count_++;
}

template<class T> void List<T>::deleteAll()
{
  T* ptr = head_;
  while (ptr) {
    T* tmp = ptr->next();
    delete ptr;
    ptr = tmp;
  }

  head_ = NULL;
  tail_ = NULL;

  current_ = NULL;
  count_ = 0;
}

template<class T> T* List<T>::pop()
{
  if (tail_) {
    T* m = tail_;
    if (tail_ != head_) {
      T* p = tail_->previous();
      p->setNext(NULL);
      tail_ = p;
      current_ = p;
      count_--;
    }
    else {
      head_ = NULL;
      tail_ = NULL;
      current_ = NULL;
      count_ = 0;
    }
    return m;
  }

  return NULL;
}

template<class T> T* List<T>::extractNext(T* ptr)
{
  T* p = ptr->previous();
  T* n = ptr->next();

  if (p)
    p->setNext(n);
  if (n)
    n->setPrevious(p);

  if (head_ == ptr)
    head_ = n;

  if (tail_ == ptr)
    tail_ = p;
  
  current_ = NULL;
  count_--;

  return n;
}

template<class T> T* List<T>::extractPrev(T* ptr)
{
  T* p = ptr->previous();
  T* n = ptr->next();

  if (p)
    p->setNext(n);
  if (n)
    n->setPrevious(p);

  if (head_ == ptr)
    head_ = n;

  if (tail_ == ptr)
    tail_ = p;
  
  current_ = NULL;
  count_--;

  return p;
}

template<class T> T* List<T>::operator[](int which)
{
  head();
  for (int i=0; i<which; i++)
    next();

  return current_;
}


template<class T> void List<T>::transverse(void (*proc)(T*))
{
  if (head())
    do
      proc(current_);
    while (next());
}
